/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package actions;

import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.Iterator;

import javax.imageio.IIOImage;
import javax.imageio.ImageIO;
import javax.imageio.ImageTypeSpecifier;
import javax.imageio.ImageWriteParam;
import javax.imageio.ImageWriter;
import javax.imageio.metadata.IIOMetadata;
import javax.imageio.metadata.IIOMetadataNode;
import javax.imageio.stream.FileImageOutputStream;
import javax.swing.ImageIcon;

import org.w3c.dom.NodeList;

import objets.CListeObjets;
import objets.CMathGrafDoc;
import types.DimensionFenetre;
import dialogs.ExporterPNGAvecUniteDlg;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import entreesSorties.MtgFileManager;

import mathgraph32.Global;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

@SuppressWarnings("serial")
public class ActionFichierExporterPNGAvecUnite extends MtgAction {
	public ActionFichierExporterPNGAvecUnite(MtgFrame frameInit) {
		super(frameInit, null, getString("chaineMenuFichierExporterPNGAvecUnite"),
				getString("chaineTipExporterUnite"), iconeAssociee(), null);
	}
	
  @Override
  public int idMenu() {
    return 33022;
  }
	
	// Modifi version 4.7.0.3 pour bug avec CEditeur et image de fond
	@Override
	public void actionPerformed(ActionEvent e) {
		MtgInternalFrame cadre = frame.cadreActif();
		if (cadre != null) {
			CListeObjets liste = cadre.listePr;
			if (liste.pointeurLongueurUnite == null) {
	    	String ch = getString("messageOutilPasDisponible") +"\n" + getString("messagePasDeLongueurUnite");
	    	frame.dialogueAvertissement(ch);
	    	frame.activeOutilCapture();
	    	return;
			}
			ExporterPNGAvecUniteDlg dlg = new ExporterPNGAvecUniteDlg(frame, true);
			dlg.setLocationRelativeTo(frame);
			dlg.setVisible(true);
			if (dlg.retourOK()) {
				File file = dlg.fileChooser.getSelectedFile();
				String path = file.getAbsolutePath();
				String chlwc = path.toLowerCase();
				if (chlwc.indexOf(Global.GLOBAL_COMMA_PNG) == -1) {
					path = path + Global.GLOBAL_COMMA_PNG;
					file = new File(path);
				}
				// On regarde si un fichier du mme nom existe dj.
				// Si oui on demande confirmation pour l'crasement
				if (!MtgFileManager.accepteEnregistrement(frame, path)) return;
				float dpi = dlg.listeChoixDPI.resolutionDPI();
				int width;
				int height;
				BufferedImage image;
				try {
					width = dlg.widthPixel;
					height = dlg.heightPixel;
					// On cre une image et on dessine la figure dedans
					image = new BufferedImage(width, height, BufferedImage.TYPE_4BYTE_ABGR);
				}
				catch (Exception er) {
					frame.dialogueAvertissement(getString("ch193"));
					return;
				}
				Graphics2D g2D = (Graphics2D) image.getGraphics();
				g2D.setColor(cadre.couleurFond); // Ncessaire car peut tre appel en dehors
				// de paint
				g2D.fillRect(0, 0, width, height);
				// Modifi version 4.6.4
		    // AffineTransform oldat = g2D.getTransform();
		    // double ratio = (double) width / cadre.dimf.longueurAbscisses/10;
		    // AffineTransform at = new AffineTransform(ratio, 0, 0, ratio, 0, 0);			
		    // g2D.setTransform(at);
				Image imageFond = cadre.document.imageFond;
				// Correction d'un bug pour la taille de l'iamge de fond version 4.7.0.3
				double coef = (double)(dlg.widthPixel)/cadre.dimf.longueurAbscisses;
				if (cadre.document.modeTraceActive) g2D.drawImage(cadre.imageTraces, 0, 0, dlg.widthPixel,
						dlg.heightPixel, cadre.couleurFond, null);
				else {
					if ((imageFond != null) && cadre.document.imageFondVisible) {
						g2D.drawImage(imageFond, 0, 0, (int)(coef*imageFond.getWidth(null)), 
								(int)(coef*imageFond.getHeight(null)), cadre.couleurFond,  null);
					}
				}
				// On change la rsolution. Pour cela on fait avant une sauvegarde de la figure dans un
				// flux afin de la recharer ensuite dans le mme tat
		    try {
		    	CMathGrafDoc doc = cadre.document;
					ByteArrayOutputStream sauvegarde = new ByteArrayOutputStream();
					MathGraphDataOutputStream oups = new MathGraphDataOutputStream(sauvegarde);
			    cadre.listePr.write(oups);
			    oups.close();
			    CListeObjets listeCopie = new CListeObjets(cadre.listePr);
			    listeCopie.bCreationComposants = false;
			    ByteArrayInputStream ba = new ByteArrayInputStream(sauvegarde.toByteArray());
			    MathGraphDataInputStream inps = new MathGraphDataInputStream(ba);
			    listeCopie.read(inps);
			    inps.close();
			    
			    listeCopie.adaptePourHauteResolution(coef);
			    DimensionFenetre dim = new DimensionFenetre(dlg.widthPixel,dlg.heightPixel);
			    listeCopie.basseDefinition = false;
			    listeCopie.bAttenuationEpaisseurTrait = dlg.cbAttenuation.isSelected();
			    listeCopie.positionne(cadre.infRand, dim);
			    listeCopie.afficheTout(g2D, dim, cadre.masquageObjetsCaches, doc.couleurFond, doc.transparencePourSurfaces, frame.pref_antialiasing);
			    listeCopie.retireTout();
		    }
		    catch (IOException err) {}
		    /*
		    finally {
					g2D.setTransform(oldat);
		    }
		    */
		    image.flush();
		    Iterator<ImageWriter> it = ImageIO.getImageWritersByFormatName(Global.GLOBAL_PNG);
		    ImageWriter writer = it.next();
		    ImageWriteParam writeParam = writer.getDefaultWriteParam();
		
		    try {		
			    FileImageOutputStream fios = new FileImageOutputStream(file);
			    writer.setOutput(fios);
			    IIOMetadata metaData = 
			    	writer.getDefaultImageMetadata(new ImageTypeSpecifier(image), writeParam);
			    IIOMetadataNode root = (IIOMetadataNode)
			        metaData.getAsTree("javax_imageio_1.0");
			    NodeList nl = root.getElementsByTagName("Dimension");
			    IIOMetadataNode nodedim;
			    if ((nl != null) && (nl.getLength() > 0))
			        nodedim = (IIOMetadataNode) nl.item(0);
			    else {
			        nodedim = new IIOMetadataNode("Dimension");
			        root.appendChild(nodedim);
			    }
			    String ch = Float.toString(dpi/25.4f);
			    nl = root.getElementsByTagName("HorizontalPixelSize");
			    if ((nl == null) || (nl.getLength() == 0)) {
			        IIOMetadataNode horz = new IIOMetadataNode("HorizontalPixelSize");
			        horz.setAttribute("value", ch);
			        nodedim.appendChild(horz);
			    }
			    nl = root.getElementsByTagName("VerticalPixelSize");
			    if ((nl == null) || (nl.getLength() == 0)) {
			        IIOMetadataNode vert = new IIOMetadataNode("VerticalPixelSize");
			        vert.setAttribute("value", ch);
			        nodedim.appendChild(vert);
			    }
				 
			    metaData.setFromTree("javax_imageio_1.0", root);	
		    
		     
			    // Ecriture du fichier image
			    	writer.write(null, new IIOImage(image, null, metaData), writeParam);
			    	fios.close();
			  }
		    catch (IOException exc) {
		    	frame.dialogueAvertissement(getString("ch45"));		    	
		    }
		    finally {
		    	writer.dispose();
		    }					
		  }
			dlg.fileChooser.setVisible(false);
			dlg.dispose();
		}
	}
	
	// Ajout version 4.6.3
  public static ImageIcon iconeAssociee() {
    URL adr = MtgFrame.class.getResource("images/ExportUnite.png");
    return new ImageIcon(adr);    
  }
	

}
