/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package actions;

import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import outils.Outil;
import mathgraph32.MtgFrame;


/**
 * Classe anctre de toutes les actions utilises dans MathGraph32.
 * Pour qu'une telle action soit utilise au moins une figure doit tre ouverte
 * @author Yves
 */
@SuppressWarnings("serial")
public abstract class MtgAction extends AbstractAction {
  public MtgFrame frame;
  public ImageIcon imicon; // Ajout version 4.3
  // Si l'action est associe  un outil, outil  pointera sur celui-ci, sinon outil sera null;
  public Outil outilAssocie;
  /**
   * Constructeur sans outil associ
   * @param frameInit le frame de l'application
   * @param chaineMenu la chane pour le menu
   * @param tip le Tooltip asoci
   * @param icon L'icne asscoi (nul si pas de bouton associ)
   * @param mnemonic Le raccouric clavier associ (nul si aucun)
   */
  public MtgAction(MtgFrame frameInit, String chaineMenu,
          String tip, ImageIcon icon, Object mnemonic) {
    super(chaineMenu, icon);
    imicon = icon;
    outilAssocie = null;
    frame = frameInit;
    if (tip != null) putValue(SHORT_DESCRIPTION, tip);
    if (mnemonic != null) putValue(MNEMONIC_KEY, mnemonic);
  }
  /**
   * 
   * @param frameInit le frame de l'application
   * @param outilAssocieInit l'outim associ
   * @param chaineMenu la chane pour le menu
   * @param tip le Tooltip asoci
   * @param icon L'icne associ (nul si pas de bouton associ)
   * @param mnemonic Le raccouric clavier associ (nul si aucun)
   */
  public MtgAction(MtgFrame frameInit, Outil outilAssocieInit, String chaineMenu,
          String tip, ImageIcon icon, Object acceleratorKey) {
    super(chaineMenu, icon);
    imicon = icon;
    outilAssocie = outilAssocieInit;
    frame = frameInit;
    if (tip != null) putValue(SHORT_DESCRIPTION, tip);
    if (acceleratorKey != null) putValue(Action.ACCELERATOR_KEY, acceleratorKey);
  }
  /**
   * Dfinition des constantes C++ associes aux items de menus
   */
  public int idMenu() {
    return 0;
  }
  
  /**
   * Fonction qui renverra true seulement pour les outils figurant par dfaut dans la barre
   * d'outils horizontale
   */
  public boolean estDansBarreHorizontale() {
  	return false;
  }
  
  public void actionPerformed(ActionEvent e) {
  	if (frame.cadreActif() == null) {
  		frame.dialogueAvertissement(getString("messagePasDeFigureOuverte"));
  		return;
  	}
  	frame.activeIconeAssocieeAction(this);
    if (outilAssocie != null) {
      if (outilAssocie.activationValide()) {
        frame.outilActif.deselect();
        frame.outilActif = outilAssocie;
        // On informe les barres d'outil de l'activation si cela a t fait via un menu
        // frame.activeIconeAssocieeAOutil(frame.outilActif);
        outilAssocie.select();
        frame.actionEnCours = this;
        // requestFocus();
      }
      else {
      	String ch = getString("messageOutilPasDisponible");
      	if (outilAssocie.messageActivationNonValide() != null) ch = ch + "\n" + outilAssocie.messageActivationNonValide();
        frame.dialogueAvertissement(ch);
        outilAssocie.deselect();
        // On ractive l'outil de capture si l'outil ne peut pas tre activ
        frame.activeOutilCapture();
      }         
    }
    frame.cadreActif().requestFocus();
  }
  
  public static final String getString(String key) {
  	return MtgFrame.getString(key);
  }
  
}
