/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.ListIterator;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import mathgraph32.MtgFrame;
import objets.CCalcul;
import objets.CCalculComplexe;
import objets.CElementBase;
import objets.CElementGenerique;
import objets.CListeObjets;
import objets.CPrototype;
import objets.CValeurAssocieeAVariable;
import objets.CValeurAssocieeComp;
import types.NatureCalcul;
import types.PointeurElementBase;
/**
 * Bote de dialogue modifie pour la verion 3.2.8.2 pour permetre, pour les valeurs sources numriques
 * autres que fonctions ou suites d'entrer une formule ou de choisir une valeur existante.
 * Si une formule est entre,  la fermeture de la bote de dialogue seront crs des calculs
 * contenant ces formules.
 * @author Yves
 *
 */
@SuppressWarnings("serial")
public class ChoixObjetsNonGraphiquesPourImplementationProtoDlg extends MtgDlg implements FocusListener{
	private CPrototype proto;
	// Modification version 3.4.0.3 : le dialogue doit mmoriser le nombre d'objets de type calculs crs
	// car si une fois ce dialogue ferm et des calculs ont t crs ne va pas  son terme, il faudra dtruire
	// les calculs crs lorsque l'outil est dslectionn
	public int nombreCalculsCrees;
	private JLabel label1, label2, label3, label4, label5, label6, label7, label8, label9;
	private JButton btnOK, btnCancel;
	private JList<String> listeIndices;
	private JList<String> listeValeurs;
	private JScrollPane scrollPaneListeIndices, scrollPaneListeValeurs;
	private JTextField infoElement;
	private JTextArea infoProto;
	private JScrollPane scrollPaneInfoProto;
	private Vector<String> vectorNumeros;
	private Vector<CElementBase> vectorValeurs;
	private Vector<PointeurElementBase> vectorPointeurs; // Contient pour chaque indice un pointeur sur la valeur associe choisie
	private Vector<CValeurAssocieeAVariable> vectorValeursAssocieesAVariable;
	private Vector<CValeurAssocieeComp> vectorValeursAssocieeComps;
	private Vector<String> vectorChainesFormules;
	private JTextField editeurFormule;
	private JButton btnListeValeurs;
	private JPanel paneEditionFormule;
	private JTextArea info;

  private DefaultListModel<String> listModel;
  int indiceSelectionnePrecedent;
  private MtgFocusTraversalPolicy mtgFocusTraversalPolicy; // Ajout version 4.7.5.2
	
  public ChoixObjetsNonGraphiquesPourImplementationProtoDlg(MtgFrame frameInit, CPrototype protoInit) {
    super (getString("implementationProtoChoixObjetsCalculsDlg1"), true, frameInit);
    proto = protoInit;
    initComponents();
    getRootPane().setDefaultButton(btnOK); 
  }
  
  private void initComponents() {
  	indiceSelectionnePrecedent = 0;
  	label1 = new JLabel(getString("implementationProtoChoixObjetsCalculsDlg2"));
  	label2 = new JLabel(getString("implementationProtoChoixObjetsCalculsDlg3"));
  	label3 = new JLabel(getString("implementationProtoChoixObjetsCalculsDlg4"));
  	label4 = new JLabel(getString("implementationProtoChoixObjetsCalculsDlg5"));
  	label5 = new JLabel(getString("implementationProtoChoixObjetsCalculsDlg6"));
  	label6 = new JLabel(getString("informationSurConstruction"));
  	label7 = new JLabel(getString("implementationProtoChoixObjetsCalculsDlg7"));
  	label8 = new JLabel(getString("implementationProtoChoixObjetsCalculsDlg8"));
  	label9 = new JLabel(getString("testExistenceDlg5"));
	  btnOK = new javax.swing.JButton(getString("btnOK"));
	  btnOK.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnOKActionPerformed();
	    }
	  });    
	  btnCancel = new javax.swing.JButton(getString("btnCancel"));
	  btnCancel.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnCancelActionPerformed();
	    }
	  });
	  btnOK.setMaximumSize(btnCancel.getPreferredSize());
	  vectorNumeros = new Vector<String>(12);
    vectorPointeurs = new Vector<PointeurElementBase>(12);
    vectorValeurs = new Vector<CElementBase>(50);
    vectorValeursAssocieesAVariable = new Vector<CValeurAssocieeAVariable>(12);
    vectorValeursAssocieeComps = new Vector<CValeurAssocieeComp>(12);
    vectorChainesFormules = new Vector<String>(12);
	  int nbSourcesCalcul = proto.nombreSourcesTypeCalcul();
	  for (int i = 0; i < nbSourcesCalcul; i++) {
	  	vectorNumeros.add(Integer.toString(i + 1));
	  	vectorPointeurs.add(new PointeurElementBase());
	  	vectorValeursAssocieesAVariable.add(new CValeurAssocieeAVariable());
	  	vectorValeursAssocieeComps.add(new CValeurAssocieeComp());
	  	vectorChainesFormules.add(new String());
	  }
	  editeurFormule = new EditeurValeurDynamique(frame, null, null);
	  editeurFormule.addKeyListener(new KeyListener() {
			public void keyTyped(KeyEvent e) {
				editeurFormuleKeyTyped();
			}
			
			public void keyReleased(KeyEvent e) {
				// TODO Auto-generated method stub
				
			}
			
			public void keyPressed(KeyEvent e) {
				// TODO Auto-generated method stub
				
			}
		});
	  btnListeValeurs = new JButton(getString("listeValeurs"));
    btnListeValeurs.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnListeValeursActionPerformed();
      }
    });
    btnOK.addFocusListener(this);
    info = new JTextArea();
    // info.setFont(FonteNom.getFonteNom(2, FonteNom.Normal, -1));   // Supprim version 4.7.5.2 pris en charge augmentation polices systme    
    info.setEditable(false);
    info.setColumns(20);
    info.setLineWrap(false);
    info.setRows(4);
    JScrollPane scrollPaneInfo = new JScrollPane(info, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    scrollPaneInfo.setPreferredSize(new Dimension(300, 90));    
	  
	  listeIndices = new JList<String>();
	  listeIndices.setListData(vectorNumeros);
    listeIndices.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    listeIndices.setVisibleRowCount(12);
    listeIndices.setSelectedIndex(0);
    listeIndices.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent evt) {
      	listeIndicesSelectionChanged();
      }
    });
    
    scrollPaneListeIndices = new JScrollPane(listeIndices, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
    		JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    listeValeurs = new JList<String>();
    listModel = new DefaultListModel<String>();
    listeValeurs.setModel(listModel);
    listeValeurs.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    listeValeurs.setVisibleRowCount(12);
    listeValeurs.setSelectedIndex(-1);
    listeValeurs.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent evt) {
      	listeValeursSelectionChanged();
      }
    });
    scrollPaneListeValeurs = new JScrollPane(listeValeurs, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
    		JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    infoElement = new JTextField();
    // infoElement.setFont(FonteNom.getFonteNom(2, FonteNom.Normal, -1)); // Supprim version 4.7.5.2 pris en charge augmentation polices systme   
    infoElement.setEditable(false);
    // infoElement.setColumns(50);
    infoProto = new JTextArea();
    // infoProto.setFont(FonteNom.getFonteNom(2, FonteNom.Normal, -1)); // Supprim version 4.7.5.2 pris en charge augmentation polices systme  
    infoProto.setEditable(false);
    infoProto.setColumns(50);
    infoProto.setRows(10);
    // On met dans infoProto la chane d'aide  l'exception de la partie affiche pour aider  la dsignation
    // c'est  dire ce qui suit le premier # s'il y en a un
    String ch = proto.commentaire;
    int j = ch.indexOf('#');
    if (j != -1) infoProto.setText(ch.substring(0, j));
    else infoProto.setText(ch);
    infoProto.setCaretPosition(0);
    scrollPaneInfoProto = new JScrollPane(infoProto, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
    		JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    
    
    JPanel panePrincipal = new JPanel();
    JPanel paneRight = new JPanel();
    panePrincipal.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    paneEditionFormule = new JPanel();
        
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 0;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    paneEditionFormule.setLayout(new GridBagLayout());
    setConstraints(constraints, 0, 0, 1, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneEditionFormule.add(label7, constraints);    
    setConstraints(constraints, 0, 1, 1, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneEditionFormule.add(label8, constraints);    
    setConstraints(constraints, 0, 2, 1, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneEditionFormule.add(editeurFormule, constraints);    
    setConstraints(constraints, 0, 3, 1, 1, 100, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneEditionFormule.add(btnListeValeurs, constraints);
    setConstraints(constraints, 0, 4, 1, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneEditionFormule.add(label9, constraints);
    setConstraints(constraints, 0, 5, 1, 1, 100, 40, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    paneEditionFormule.add(scrollPaneInfo, constraints);
    
    
    setConstraints(constraints, 0, 0, 3, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label1, constraints);    
    setConstraints(constraints, 0, 1, 3, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label2, constraints);    
    constraints.insets.bottom = 5;
    setConstraints(constraints, 0, 2, 1, 1, 30, 10, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    panePrincipal.add(label3, constraints);
    setConstraints(constraints, 1, 2, 1, 1, 30, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    panePrincipal.add(label4, constraints);    
    setConstraints(constraints, 0, 3, 1, 1, 0, 40, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    panePrincipal.add(scrollPaneListeIndices, constraints);    
    setConstraints(constraints, 1, 3, 1, 1, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    panePrincipal.add(scrollPaneListeValeurs, constraints);
    setConstraints(constraints, 2, 3, 2, 1, 30, 0, GridBagConstraints.NONE, GridBagConstraints.NORTHWEST);
    panePrincipal.add(paneEditionFormule, constraints);
    
    setConstraints(constraints, 0, 4, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label5, constraints);    
    setConstraints(constraints, 1, 4, 3, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    panePrincipal.add(infoElement, constraints);    
    setConstraints(constraints, 0, 5, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label6, constraints);    
    setConstraints(constraints, 0, 6, 4, 1, 0, 40, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(scrollPaneInfoProto, constraints);    
    
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
    setConstraints(constraints, 3, 0, 1, 2, 0, 0, GridBagConstraints.NONE, GridBagConstraints.NORTHEAST);
    panePrincipal.add(paneRight, constraints);
    // Ajout version 4.7.5.2
    Vector<Component> order = new Vector<Component>(5);
    order.add(listeIndices);
    order.add(listeValeurs);
    order.add(editeurFormule);
    order.add(btnOK);
    order.add(btnCancel);
    mtgFocusTraversalPolicy = new MtgFocusTraversalPolicy(order);
    setFocusTraversalPolicy(mtgFocusTraversalPolicy);
    // Fin ajout
    
    setContentPane(panePrincipal);
    pack(); 
    listeIndicesSelectionChanged();
  }
	
  private void btnOKActionPerformed() {
  	vectorChainesFormules.set(indiceSelectionnePrecedent,editeurFormule.getText());
  	nombreCalculsCrees = 0;
  	// On regarde si on a bien affect un objet  chaque objet source
  	int nb = vectorNumeros.size();
  	for (int i = 0; i < nb; i ++) {
  		PointeurElementBase p = vectorPointeurs.get(i);
  		if (p.getPointeur() == null) {
  			if (vectorChainesFormules.get(i).isEmpty()) {
	  			listeIndices.setSelectedIndex(i);
	  			listeIndicesSelectionChanged();
	  			dialogueAvertissement(getString("ch136") + " " + (Integer.toString(i + 1) + "\n" + getString("ch137")));
	  			return;
  			}
  			else {
  				// Si on a entr une formule pour un champ numrique, on vrifie s'il est correct
  		  	CElementGenerique gen = (CElementGenerique) proto.get(i);
  		  	long nat = CElementBase.natureObjetNonGraphiquePourImplementationProto(gen.natureCalcul);
  		  	if ((nat & NatureCalcul.NTouteValeurReelle) != 0) {
  		  		EditeurValeurDynamique editeur = new EditeurValeurDynamique(frame, vectorValeursAssocieesAVariable.get(i), null);
  		  		editeur.associeA(vectorValeursAssocieesAVariable.get(i));
  		  		editeur.setText(vectorChainesFormules.get(i));
  		  		if (!editeur.verifieValidite()) {
  		  			listeIndices.setSelectedIndex(i);
  		  			editeurFormule.setCaretPosition(editeur.getCaretPosition());
  		  			editeurFormule.requestFocus();
  		  			return;
  		  		}
  		  	}
  		  	else {
  		  		EditeurValeurComplexe editeur = new EditeurValeurComplexe(frame, vectorValeursAssocieeComps.get(i), null);
  		  		editeur.associeA(vectorValeursAssocieeComps.get(i));
  		  		editeur.setText(vectorChainesFormules.get(i));
  		  		if (!editeur.verifieValidite()) {
  		  			listeIndices.setSelectedIndex(i);
  		  			editeurFormule.setCaretPosition(editeur.getCaretPosition());
  		  			editeurFormule.requestFocus();
  		  			return;
  		  		}
  		  	}
  		  	nombreCalculsCrees ++;
  			}
  		}
  	}
  	// Si on arrive ici tout est correct
  	// On met dans les lments gnriques correspondant les pointeurs sur les valeurs choisies
  	// Ou si une formule a t entre on cre un calcul
  	for (int i = 0; i < nb; i++) {
  		CElementGenerique gen = (CElementGenerique) proto.get(i);
  		PointeurElementBase p = vectorPointeurs.get(i);
  		if (p.getPointeur() != null) gen.elementAssocie = p.getPointeur();
  		else {
		  	long nat = CElementBase.natureObjetNonGraphiquePourImplementationProto(gen.natureCalcul);
		  	CListeObjets liste = frame.cadreActif().listePr;
		  	String nom = liste.genereNomPourCalcul(gen.nom, false);
		  	if ((nat & NatureCalcul.NTouteValeurReelle) != 0) {
		  		CCalcul calcul = new CCalcul(liste, null, false, nom,vectorChainesFormules.get(i),
		  				vectorValeursAssocieesAVariable.get(i).calcul);
		  		frame.ajouteElement(calcul);
		  		gen.elementAssocie = calcul;
		  	}
		  	else {
		  		CCalculComplexe calcul = new CCalculComplexe(liste, null, false, nom,vectorChainesFormules.get(i),
		  				vectorValeursAssocieeComps.get(i).calcul);
		  		frame.ajouteElement(calcul);
		  		gen.elementAssocie = calcul;		
		  	}
  		}
  	}
  	setRetourOK(true);
  	dispose();
  }

  private void btnCancelActionPerformed() {
  	dispose();
  	return;
  }
  
  public void listeIndicesSelectionChanged() {
  	String formule = editeurFormule.getText();
  	vectorChainesFormules.set(indiceSelectionnePrecedent, formule);
  	if (!formule.isEmpty()) vectorPointeurs.get(indiceSelectionnePrecedent).setPointeur(null);
  	CListeObjets liste = frame.cadreActif().listePr;
  	int ind = listeIndices.getSelectedIndex();
  	CElementGenerique gen = (CElementGenerique) proto.get(ind);
  	long nat = CElementBase.natureObjetNonGraphiquePourImplementationProto(gen.natureCalcul);
  	if ((nat & NatureCalcul.NTouteValeurReelleOuComplexe) != 0) {
  		editeurFormule.setEditable(true);
  		editeurFormule.requestFocus();
  		btnListeValeurs.setVisible(true);
  	}
  	else {
  		editeurFormule.setEditable(false);
  		btnListeValeurs.setVisible(false);
  	}
		vectorValeurs.removeAllElements();
		listModel.removeAllElements();
		ListIterator<CElementBase> li = liste.listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (!elb.estElementIntermediaire() && elb.estDeNatureCalcul(nat)) {
				listModel.addElement(elb.nom());
				vectorValeurs.addElement(elb);
			}
		}
  	if (vectorChainesFormules.get(ind).isEmpty()) {
	  	CElementBase el = vectorPointeurs.get(ind).getPointeur();
  		editeurFormule.setText("");
	  	if (el != null) {
	  		int indice = vectorValeurs.indexOf(el);
	  		listeValeurs.setSelectedIndex(indice);
	  		listeValeurs.ensureIndexIsVisible(indice);
	  		info.setText(el.infoPourHistorique());
	  	}
	  	else {
	  		listeValeurs.setSelectedIndex(-1);
	  		if (!info.getText().isEmpty()) info.setText("");
	  	}
  	}
  	else {
  		editeurFormule.setText(vectorChainesFormules.get(ind));
  		if (!info.getText().isEmpty()) info.setText("");
  	}
  	if (vectorValeurs.size() == 0) scrollPaneListeValeurs.setVisible(false);
  	else scrollPaneListeValeurs.setVisible(true);
  	// Modification version 3.1.5 : Si rien n'a t prcis dans le commentaire, on prcise le type d'objet attendu
  	String ch = proto.chaineImplementationProto(ind + 1);
  	if (ch.length() != 0) infoElement.setText(ch);
  	else infoElement.setText(CElementBase.chaineNatureCalculPourImplementationProto(nat));
  	indiceSelectionnePrecedent = ind;
  	pack();
  }
  
  public void listeValeursSelectionChanged() {
  	int ind = listeValeurs.getSelectedIndex();
  	if (ind != -1) {
  		int indice = listeIndices.getSelectedIndex();
  		PointeurElementBase pointeur = vectorPointeurs.get(indice);
  		pointeur.setPointeur(vectorValeurs.get(ind));
  		editeurFormule.setText("");
  		vectorChainesFormules.set(indice,"");
  		info.setText(vectorValeurs.get(ind).infoPourHistorique());
  		pack();
  	}
  }
 
  private void editeurFormuleKeyTyped() {
  	listeValeurs.clearSelection();
  	String formule = editeurFormule.getText();
  	if (!formule.isEmpty()) vectorPointeurs.get(indiceSelectionnePrecedent).setPointeur(null);
  }
    
  private void btnListeValeursActionPerformed() {
  	int indmax = frame.cadreActif().listePr.size() - 1;
  	int ind = listeIndices.getSelectedIndex();
  	CElementGenerique gen = (CElementGenerique) proto.get(ind);
  	long nat = CElementBase.natureObjetNonGraphiquePourImplementationProto(gen.natureCalcul);
  	if ((nat & NatureCalcul.NTouteValeurReelle) != 0) {
			ListeValeursOuFonctionsReellesTextFieldDlg dlg = 
				new ListeValeursOuFonctionsReellesTextFieldDlg(frame, editeurFormule, indmax, false);
			dlg.setLocationRelativeTo(frame); // Modifi version 4.7.5.2
			dlg.setVisible(true);  		
  	}
  	else {
			ListeValeursOuFonctionsReellesOuComplexesTextFieldDlg dlg = 
				new ListeValeursOuFonctionsReellesOuComplexesTextFieldDlg(frame, editeurFormule, indmax, false);
			dlg.setLocationRelativeTo(frame); // Modifi version 4.7.5.2
			dlg.setVisible(true);
  	}
  }
  
  // Nouveau version 3.5.1 : Si le bouton btnListeValeurs gagne le focus, c'est qu'on
  // a appuy sur Tab dans l'diteur de formule et alors on active le numro
  // suivant si c'est possible
	public void focusGained(FocusEvent arg0) {
		// Si c'est l'diteur de formule qui perd le focus on active si possible l'lment suivant de la liste
		if (arg0.getSource() == btnOK) {
			int ind = listeIndices.getSelectedIndex();
			if (ind != vectorNumeros.size() - 1) {
				listeIndices.setSelectedIndex(ind+1);
			}
		}
		
	}

	public void focusLost(FocusEvent arg0) {
	}
   	
}
