/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;
import java.awt.*;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import objets.CValeurDynamique;
import types.NatureCalcul;
import java.awt.event.*;
import mathgraph32.MtgFrame;


/**
 *
 * @author Yves
 * Ce type de dialogue sera ouvert lorsqu'on clique sur un bouton Liste des valeurs
 * dans une boite de dialogue permettant de spcifier un angle
 */
@SuppressWarnings("serial")
public class ListeValeursComboDlg extends MtgDlg implements ActionListener,
    MouseListener {
  private EditeurAngle editeur;   // Le combo d'entre d'angle associ
  @SuppressWarnings("unused")
	private int indiceMaxi;         // L'indice maxi dans la lsite d'objets du dernier objet qui puiss
                          // figurer dans le combo
  private boolean intermPermis; // True si les objets intermdiaires sont autoriss dans la liste
  private ListeValeursOuFonctionsReelles listeVal;
  private JButton btnOK, btnCancel;  
  private JTextArea info;    
	private JLabel label;
  
  /** Creates a new instance of ListeValeursComboDlg */
  public ListeValeursComboDlg(MtgFrame frameInit, EditeurAngle editeurInit,
      int indiceMaxiInit, boolean intermPermisInit) {
    super("Choix d'une valeur existante ",true, frameInit);
    this.setResizable(false);
    label = new JLabel(getString("listeValeursDlg2"));
    editeur = editeurInit;
    indiceMaxi = indiceMaxiInit;
    intermPermis = intermPermisInit;
    btnOK = new JButton(getString("btnOK"));
    btnCancel = new JButton(getString("btnCancel"));
    btnOK.setMaximumSize(btnCancel.getPreferredSize());
    info = new JTextArea();
    // info.setFont(FonteNom.getFonteNom(2, FonteNom.Normal, -1)); // Supprim version 4.7.5.2 pris en charge augmentation polices systme      
    JScrollPane scrollPane = new JScrollPane(info, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    info.setEditable(false);
    info.setColumns(20);
    info.setEditable(false);
    info.setLineWrap(false);
    info.setRows(4);
    scrollPane.setPreferredSize(new Dimension(350, 90));    
    listeVal = new ListeValeursOuFonctionsReelles(frame.cadreActif().listePr, indiceMaxiInit, intermPermis);
    JPanel pane = new JPanel();
    GridBagLayout gridBag = new GridBagLayout();
    pane.setLayout(gridBag);
    GridBagConstraints constraints = new GridBagConstraints();
    setConstraints(constraints, 0, 0, 1, 1, 80, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    constraints.insets.top = 4;
    constraints.insets.bottom = 4;
    constraints.insets.left = 5;
    constraints.insets.right = 5;

    gridBag.setConstraints(label, constraints);
    pane.add(label);
    constraints.insets.top = 0;
    constraints.insets.bottom = 5;    
    setConstraints(constraints, 0, 1, 1, 1, 0, 70, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    JScrollPane scrollList = new JScrollPane(listeVal, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    
    //constraints.insets.bottom = 60;
    gridBag.setConstraints(scrollList, constraints);
    pane.add(scrollList);
    
    
    setConstraints(constraints, 0, 2, 2, 1, 100, 30, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER);
    gridBag.setConstraints(scrollPane, constraints);
    pane.add(scrollPane);
    
    constraints.insets.bottom = 0;
    
    JPanel paneDroit = new JPanel(); // Pour mettre les deux boutons en haut et  droite
    paneDroit.setLayout(new BoxLayout(paneDroit,BoxLayout.Y_AXIS));
    paneDroit.add(btnOK);
		paneDroit.add(Box.createRigidArea(new Dimension(0, 5))); 		
    paneDroit.add(btnCancel);
    
    setConstraints(constraints, 1, 1, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.NORTHEAST);
    gridBag.setConstraints(paneDroit, constraints);
    pane.add(paneDroit);

    setContentPane(pane);
    btnOK.addActionListener(this);
    btnCancel.addActionListener(this);
    listeVal.addMouseListener(this);
    listeVal.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent evt) {
      	listValSelectionChanged();
      }
    });
    
    getRootPane().setDefaultButton(btnOK);
    listValSelectionChanged();    
    pack();  
  }

  public void actionPerformed(ActionEvent evt) {
    if (evt.getSource()== btnOK) {
      actionButtonOK();
    }
    else {
      if (evt.getSource()== btnCancel) {
        dispose();
      }
    }   
  }
  
  public void actionButtonOK() {
    String ch2;
    JTextField textField = (JTextField) editeur.getEditor().getEditorComponent();
		int i = textField.getCaretPosition();
    String ch = textField.getText();
    CValeurDynamique val = listeVal.valeurSelectionnee();
    String nom = val.nom();
    int longNom = nom.length();
  	// Si on colle le nom d'une fonction on colle aussi la parenthse
  	// et les virgules ventuelles
    int decalage = 0;
  	if (val.estDeNatureCalcul(NatureCalcul.NFonction | NatureCalcul.NDerivee | NatureCalcul.NSuiteRecurrenteReelle)) {
  		ch2 = nom + "()";
  		decalage = 1;
  	}
  	else {
    	if (val.getNatureCalcul() == NatureCalcul.NFonction2Var) {
    		ch2 = nom + "(,)";
    		decalage = 1;
    	}
    	else {
      	if (val.getNatureCalcul() == NatureCalcul.NFonction3Var) {
      		ch2 = nom + "(,,)";
      		decalage = 1;
      	}
      	else{
      		ch2 = nom;
      	}
    	}
  	}
  	textField.setText(ch.substring(0, i) + ch2 + ch.substring(i));
  	textField.setCaretPosition(longNom + decalage + i);
  	textField.requestFocus();
    dispose();  	
  }
  
  
  public void listValSelectionChanged() {
  	int i = listeVal.getSelectedIndex();
  	if (i != -1) {
  		info.setText(listeVal.pointeurs.get(i).infoPourHistorique());
  		info.setCaretPosition(0);
  		pack();
  	}
  }  

  public void mouseClicked(MouseEvent e) {
    if (e.getClickCount() == 2) {
    	actionButtonOK();
	    dispose();
    }
  }

  public void mouseEntered(MouseEvent e) {
  }

  public void mouseExited(MouseEvent e) {
  }

  public void mousePressed(MouseEvent e) {
  }

  public void mouseReleased(MouseEvent e) {
  }

}
