/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import objets.CValeurDynamique;
import types.NatureCalcul;

import mathgraph32.MtgFrame;

@SuppressWarnings("serial")
/**
 * Dialogue associ  un JTextfield, prsentant les valeurs complexes ou relles jusqu' indiceMaxi.
 * Lorsqu'on quitte la valeur choisie est insre au point d'insertion du JTextField utilis
 */
public class ListeValeursOuFonctionsReellesOuComplexesTextFieldDlg extends MtgDlg implements ActionListener, MouseListener {
  private JTextField editeur;   // Le champ d'entre associ
  @SuppressWarnings("unused")
	private int indiceMaxi;         // L'indice maxi dans la liste d'objets du dernier objet qui puiss
                          // figurer dans le combo
  private boolean intermPermis; // True si les objets intermdiaires sont autoriss dans la liste
  private JLabel label;
  private ListeValeursOuFonctionsReellesOuComplexes listeVal;
  private JButton btnOK, btnCancel;
  private JTextArea info;
  
  
  public ListeValeursOuFonctionsReellesOuComplexesTextFieldDlg(MtgFrame frameInit, JTextField editeurInit,
      int indiceMaxiInit, boolean intermPermisInit) {
    super(getString("listeValeursDlg1"),true, frameInit);
    editeur = editeurInit;
    indiceMaxi = indiceMaxiInit;
    intermPermis = intermPermisInit;
    label = new JLabel(getString("listeValeursDlg3"));
    btnOK = new JButton(getString("btnOK"));
    btnCancel = new JButton(getString("btnCancel"));
    btnOK.setMaximumSize(btnCancel.getPreferredSize());
    info = new JTextArea();
    // info.setFont(FonteNom.getFonteNom(2, FonteNom.Normal, -1)); // Supprim version 4.7.5.2 pris en charge augmentation polices systme      
    JScrollPane scrollPane = new JScrollPane(info, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    info.setEditable(false);
    info.setColumns(20);
    info.setEditable(false);
    info.setLineWrap(false);
    info.setRows(4);
    scrollPane.setPreferredSize(new Dimension(300, 90));    
    listeVal = new ListeValeursOuFonctionsReellesOuComplexes(frame.cadreActif().listePr, indiceMaxiInit, intermPermis);
    JPanel pane = new JPanel();
    GridBagLayout gridBag = new GridBagLayout();
    pane.setLayout(gridBag);
    GridBagConstraints constraints = new GridBagConstraints();
    setConstraints(constraints, 0, 0, 1, 1, 80, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    constraints.insets.top = 4;
    constraints.insets.bottom = 4;
    constraints.insets.left = 5;
    constraints.insets.right = 5;

    gridBag.setConstraints(label, constraints);
    pane.add(label);
    constraints.insets.top = 0;
    constraints.insets.bottom = 5;    
    setConstraints(constraints, 0, 1, 1, 1, 0, 70, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    JScrollPane scrollList = new JScrollPane(listeVal, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    
    //constraints.insets.bottom = 60;
    gridBag.setConstraints(scrollList, constraints);
    pane.add(scrollList);
    
    
    setConstraints(constraints, 0, 2, 2, 1, 100, 30, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER);
    gridBag.setConstraints(scrollPane, constraints);
    pane.add(scrollPane);
    
    constraints.insets.bottom = 0;
    
    JPanel paneDroit = new JPanel(); // Pour mettre les deux boutons en haut et  droite
    paneDroit.setLayout(new BoxLayout(paneDroit,BoxLayout.Y_AXIS));
    paneDroit.add(btnOK);
    paneDroit.add(btnCancel);
    
    setConstraints(constraints, 1, 1, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.NORTHEAST);
    gridBag.setConstraints(paneDroit, constraints);
    pane.add(paneDroit);

    setContentPane(pane);
    btnOK.addActionListener(this);
    btnCancel.addActionListener(this);
    listeVal.addMouseListener(this);
    listeVal.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent evt) {
      	listValSelectionChanged();
      }
    });
    
    getRootPane().setDefaultButton(btnOK);      
    listValSelectionChanged();    
    pack();
  }

  public void actionPerformed(ActionEvent evt) {
    if (evt.getSource()== btnOK) {
      actionButtonOK();
    }
    else {
      if (evt.getSource()== btnCancel) {
        dispose();
      }
    }   
  }
  
  public void listValSelectionChanged() {
  	int i = listeVal.getSelectedIndex();
  	if (i != -1) {
  		info.setText(listeVal.pointeurs.get(i).infoPourHistorique());
  		info.setCaretPosition(0);
  		pack();
  	}
  }

  public void actionButtonOK() {
		int i = editeur.getCaretPosition();
    String ch = editeur.getText();
    CValeurDynamique val = listeVal.valeurSelectionnee();
    String nom = val.nom();
    int longNom = nom.length();
    String chdebut = nom;
    String chfin;
  	// Si on colle le nom d'une fonction on colle aussi la parenthse
  	// et les virgules ventuelles
    int decalage = 0;
    boolean fonction = true;
  	if (val.estDeNatureCalcul(NatureCalcul.NFonction | NatureCalcul.NDerivee | NatureCalcul.NSuiteRecurrenteComplexe
  			| NatureCalcul.NFonctionComplexe)) {
  		chdebut = chdebut + "(";
  		chfin = ")";
  		decalage++;
  	}
  	else {
    	if (val.estDeNatureCalcul(NatureCalcul.NFonction2Var | NatureCalcul.NFonctionComplexe2Var)) {
    		chdebut = chdebut + "(";
    		chfin = ",)";
    		decalage++;
    	}
    	else {
      	if ((val.getNatureCalcul() & (NatureCalcul.NFonction3Var | NatureCalcul.NFonctionComplexe3Var)) != 0) {
       		chdebut = chdebut + "(";
      		chfin = ",,)";
      		decalage++;
       	}
      	else{
      		chfin = "";
      		fonction = false;
      	}
    	}
  	}
		String selection = editeur.getSelectedText();
		if (selection != null) {
			i = ch.indexOf(selection);
			if (fonction) {
				decalage = decalage + selection.length()+ 1;
				editeur.setText(ch.substring(0,editeur.getSelectionStart()) + chdebut + selection
					+ chfin + ch.substring(editeur.getSelectionEnd()));
			}
			else editeur.setText(ch.substring(0,editeur.getSelectionStart()) + chdebut
					+ chfin + ch.substring(editeur.getSelectionEnd()));
		}
		else {
	    String ch2 = ch.substring(0, i) + chdebut + chfin + ch.substring(i);
	    editeur.setText(ch2);
		}
	  editeur.setCaretPosition(longNom + i + decalage);
	  editeur.requestFocus();
		dispose(); 
  }
  
  public void mouseClicked(MouseEvent evt) {
    if (evt.getClickCount() == 2) {
      actionButtonOK();
    }
  }

  public void mouseEntered(MouseEvent evt) {
  }

  public void mouseExited(MouseEvent evt) {
  }

  public void mousePressed(MouseEvent evt) {
  }

  public void mouseReleased(MouseEvent evt) {
  }
}
