/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;

import javax.swing.*;

import types.Bool;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

/**
 *
 * @author Yves
 * Ancetre de tous les dialogues utiliss dans l'applet
 */
@SuppressWarnings("serial")
public class MtgDlg extends JDialog implements ComponentListener {
  protected MtgFrame frame;
  private Bool retourOK; // Contient true si la boite de dialogue a t cre en cliquant
                  // sur OK et false sinon
  public boolean getRetourOK() {
    return retourOK.getValue();
  }
  public void setRetourOK(boolean value) {
    retourOK.setValue(value);
  }
  
  /** Creates a new instance of MtgDlg */
  public MtgDlg(String title, boolean modal, MtgFrame frameInit) {
    super(frameInit, title, modal);
    frame = frameInit;
    retourOK = new Bool(false);
    setResizable(false);
    addComponentListener(this);
  }
  /** Fonction affichant un dialogue d'avertissement au centre de l'cran
   * A utiliser le moins mossible
   * Utiliser plutt CFrame.dialogueAvertissement qui centre le message par rapport
   *  la fentre de l'application
   */
  public final static void dialogueAvertissementCentreEcran(String texte) {
    JOptionPane.showMessageDialog(null, texte, getString("avertissement"), JOptionPane.WARNING_MESSAGE);
  }

	public void dialogueAvertissement(String texte) {
		frame.dialogueAvertissement(texte);
	}
	
	public void dialogueInformation(String texte) {
		frame.dialogueInformation(texte);
	}
	
	public boolean dialogueConfirmation_YES_NO(String texte) {
		return frame.dialogueConfirmation_YES_NO(texte);
	}
  
  public int dialogueConfirmation_YES_NO_CANCEL(String texte) {
  	return frame.dialogueConfirmation_YES_NO_CANCEL(texte);
  }
	
  
  /**
   * Fonction permettant de spcifier les champs d'un GridBagConstraints qui permet de
   * spcifier comment sont agencs les composants d'une boite de dialogue
   **/
  public static final void setConstraints(GridBagConstraints gbc, int gx, int gy, int gw, int gh, 
    int wx, int wy, int fill, int anchor) {
    gbc.gridx = gx;
    gbc.gridy = gy;
    gbc.gridwidth = gw;
    gbc.gridheight = gh;
    gbc.anchor = anchor;
    gbc.fill = fill;
  }
  
  /* Renvoie la valeur contenue dans retourOK ce qui permet de savoir si la boite
   * de dialogue a t ferme en validant
  */
  public final boolean retourOK() {
    return retourOK.getValue();
  }
	public void actionPerformed(ActionEvent evt) {
		// TODO Auto-generated method stub
		return;
	}
	// Redfinition de dispose() pour redonner le focus clavier  la fentre active quand
	// une fentre se referme de faon  pouvoir intercepter l'appui de F9 pour refaire la dernire action
	@Override
	public void dispose() {
		// TODO Auto-generated method stub
		MtgInternalFrame cadre = frame.cadreActif();
		if (cadre != null) cadre.requestFocus();
		System.gc(); // Passage du garbage collector
		super.dispose();
	}
	
	/**
	 * Fonction qui sera redfinie par CCommentaireDlg et CLatexDlg pour permettre
	 * de mettre  jour l'affichage clignotant quand un caractre spcial a t isr dans la formule
	 * en cliquant sur un bouton MtgCharButton
	 */
	public void metAJourAffichageClignotant() {
		
	}
	public void componentHidden(ComponentEvent e) {
		// TODO Auto-generated method stub
		
	}
	
	public void componentMoved(ComponentEvent e) {
		// Dans le cas d'une bote de dialogue ayant pour fille une CaracteresSpeciauxDlg
		// ou une CaracteresSpeciauxLatexDlg on dplace ces fentres filles relativement au dialogue.
		Window[] wind = getOwnedWindows();
		for (int i =0; i < wind.length; i++) {
			if (wind[i].getClass() == CaracteresSpeciauxDlg.class) {
				CaracteresSpeciauxDlg dlg = (CaracteresSpeciauxDlg) wind[i];
				dlg.positionneParRapportDialoguePere();
			}
			else {
				if (wind[i].getClass() == CaracteresSpeciauxLatexDlg.class) {
					CaracteresSpeciauxLatexDlg dlg = (CaracteresSpeciauxLatexDlg) wind[i];
					dlg.positionneParRapportDialoguePere();
				}
			}
		}
		
	}
	public void componentResized(ComponentEvent e) {
		// TODO Auto-generated method stub
		
	}
	public void componentShown(ComponentEvent e) {
		// TODO Auto-generated method stub
		
	}
	
	/**
	 * Version 3.2.9 : renverra true si le dialogue est tellement petit qu'un CaracteresSpeciauxDlg
	 * ne devra pas prendre pour hauteur sa moiti
	 * Redfini par EntreeNomDialog
	 * @return
	 */
	public boolean estDePetiteHauteur() {
		return false;
	}
	
	/**
	 * Version 3.2.9 : renverra true si le dialogue est centr
	 * Dans ce cas les botes de dialogue CaracteresSpeciauxDlg et
	 * et CaracteresSpeciauxLatexDlg seront dcales de faon  ne pas recouvrir le dialogue
	 * Redfini pour CommentaireDlg et LatexDlg
	 * @return
	 */
	public boolean estCentreSurFenetre() {
		return true;
	}
  
  public static final String getString(String key) {
  	return MtgFrame.getString(key);
  }
	
}
