/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionListener;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;
import objets.CListeObjets;
import objets.CRepere;
import objets.CValeurAssocieeAVariable;
import types.StyleTrait;

import mathgraph32.MtgColorButton;
import mathgraph32.MtgFrame;
import mathgraph32.MtgListeStyleTrait;


@SuppressWarnings("serial")
public class RepereDlg extends MtgDlg implements ActionListener {
	private boolean modification; // True si on modifie un repre dj cr
  private CRepere repere; // Le repre en cours de cration ou modification
  private EditeurValeurDynamique editeurOrigineAbscisses;
  private EditeurValeurDynamique editeurOrigineOrdonnees;
  // Version 4.0 : choix d'une unit pour chacun des axes
  private EditeurValeurPositiveStrict editeurUnitex;
  private EditeurValeurPositiveStrict editeurUnitey;
  
  private JButton btnOK, btnCancel;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsValAbs;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsValOrd;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsValUnitex;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsValUnitey;

  public JCheckBox checkQuadVer;
  public JCheckBox checkQuadHor;
  public JCheckBox checkPointilles;
  public JCheckBox checkCreerAxes;
  private CValeurAssocieeAVariable valOrigineAbscisses;
  private CValeurAssocieeAVariable valOrigineOrdonnees;
  private CValeurAssocieeAVariable valUnitex;
  private CValeurAssocieeAVariable valUnitey; 
  private MtgColorButton colorButton; // Bouton pour le choix de la couleur du quadrillage
  private MtgListeStyleTrait listeStyleTrait; // Choix du style de trait pour le quadrillage
 
  
  public RepereDlg(MtgFrame frameInit, CRepere repereInit, boolean modificationInit) {
    super(getString("repereDlg1"), true, frameInit);
    modification = modificationInit;
    setResizable(false);
    repere = repereInit;
    valOrigineAbscisses = new CValeurAssocieeAVariable(repereInit.abscisseOrigine);
    valOrigineOrdonnees = new CValeurAssocieeAVariable(repereInit.ordonneeOrigine);
    valUnitex = new CValeurAssocieeAVariable(repereInit.unite_x);
    valUnitey = new CValeurAssocieeAVariable(repereInit.unite_y);
   
    checkQuadVer = new JCheckBox(getString("repereDlg4"));
    checkQuadVer.setSelected(repere.quadrillageHorizontal);
    checkQuadHor = new JCheckBox(getString("repereDlg5"));
    checkQuadHor.setSelected(repere.quadrillageVertical);
    checkPointilles = new JCheckBox(getString("repereDlg6"));
    checkPointilles.setSelected(repere.pointilles);
    checkCreerAxes = new JCheckBox(getString("repereDlg9"));
    
    JLabel label1 = new JLabel(getString("repereDlg2"), SwingConstants.LEFT);
    JLabel label2 = new JLabel(getString("repereDlg3"), SwingConstants.LEFT);
    JLabel label3 = new JLabel(getString("repereDlg7"), SwingConstants.LEFT);
    JLabel label4 = new JLabel(getString("repereDlg8"), SwingConstants.LEFT);
    JLabel label5 = new JLabel(getString("repereDlg10"), SwingConstants.LEFT);
    JLabel label6 = new JLabel(getString("repereDlg11"), SwingConstants.LEFT);
    
    colorButton = new MtgColorButton(frame);
    
    listeStyleTrait = new MtgListeStyleTrait(frame);
    JScrollPane scrollpane = new JScrollPane(listeStyleTrait, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

    editeurOrigineAbscisses = new EditeurValeurDynamique(frame, valOrigineAbscisses, repere);
    editeurOrigineAbscisses.setText(repere.abscisseOrigine.chaineCalculValeur());
    editeurOrigineOrdonnees = new EditeurValeurDynamique(frame, valOrigineOrdonnees, repere);
    editeurOrigineOrdonnees.setText(repere.ordonneeOrigine.chaineCalculValeur());
    editeurUnitex = new EditeurValeurPositiveStrict(frame, valUnitex, repere);
    editeurUnitex.setText(repere.unite_x.chaineCalculValeur());
    editeurUnitey = new EditeurValeurPositiveStrict(frame, valUnitey, repere);
    editeurUnitey.setText(repere.unite_y.chaineCalculValeur());
    
		btnOK = new javax.swing.JButton(getString("btnOK"));
		btnOK.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				btnOKActionPerformed();
			}
		});
		btnCancel = new javax.swing.JButton(getString("btnCancel"));
		btnCancel.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				btnCancelActionPerformed();
			}
		});
		// btnOK.setMaximumSize(btnCancel.getPreferredSize());
		btnOK.setPreferredSize(btnCancel.getPreferredSize());
    CListeObjets liste = frame.cadreActif().listePr;
    int indmax;
    if (modification) indmax = liste.indexOf(repere) - 1;
    else indmax = liste.size() - 1;
    paneListeValeurEtFonctionsValAbs = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurOrigineAbscisses, true, indmax, false);
    paneListeValeurEtFonctionsValOrd = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurOrigineOrdonnees, true, indmax, false);
    paneListeValeurEtFonctionsValUnitex = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurUnitex, true, indmax, false);
    paneListeValeurEtFonctionsValUnitey = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurUnitey, true, indmax, false);
   
    
    if (modification) {
    	colorButton.couleur = repere.couleur;
    	listeStyleTrait.setSelectedIndex(repere.style);
    	checkQuadHor.setSelected(repere.quadrillageHorizontal);
    	checkQuadVer.setSelected(repere.quadrillageVertical);
    	checkPointilles.setSelected(repere.pointilles);
    	editeurOrigineAbscisses.setText(repere.abscisseOrigine.chaineInfo());
    	editeurOrigineOrdonnees.setText(repere.ordonneeOrigine.chaineInfo());
    	editeurUnitex.setText(repere.unite_x.chaineInfo());
    	editeurUnitey.setText(repere.unite_y.chaineInfo());
    	checkCreerAxes.setVisible(false);
    }
    else {
    	listeStyleTrait.setSelectedIndex(StyleTrait.traitPointille);
    	checkCreerAxes.setSelected(true);
    }
    
    
    JPanel pane = new JPanel();
    pane.setLayout(new GridBagLayout());

    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 2;
    constraints.insets.top = 0;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 30, 14, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(checkQuadVer, constraints);
    constraints.insets.bottom = 5;
    setConstraints(constraints, 0, 1, 1, 1, 0, 14, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(checkQuadHor, constraints);
    setConstraints(constraints, 0, 2, 1, 1, 0, 14, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(checkPointilles, constraints);
    setConstraints(constraints, 1, 0, 1, 1, 30, 14, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label3, constraints);
    setConstraints(constraints, 2, 0, 1, 1, 10, 14, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(colorButton, constraints);
    setConstraints(constraints, 1, 1, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label4, constraints);
    setConstraints(constraints, 2, 1, 1, 2, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(scrollpane, constraints);
    
    setConstraints(constraints, 0, 3, 1, 1, 0, 14, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label1, constraints);
    constraints.insets.bottom = 2;
    setConstraints(constraints, 0, 5, 1, 1, 0, 14, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label2, constraints);
    setConstraints(constraints, 1, 3, 2, 1, 30, 14, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(editeurOrigineAbscisses, constraints);
    setConstraints(constraints, 1, 4, 1, 1, 0, 14, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(paneListeValeurEtFonctionsValAbs, constraints);
    setConstraints(constraints, 1, 5, 2, 1, 30, 14, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(editeurOrigineOrdonnees, constraints);
    setConstraints(constraints, 1, 6, 1, 1, 0, 14, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(paneListeValeurEtFonctionsValOrd, constraints);
    setConstraints(constraints, 0, 7, 1, 1, 0, 14, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label5, constraints);
    setConstraints(constraints, 1, 7, 2, 1, 30, 14, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(editeurUnitex, constraints);
    setConstraints(constraints, 1, 8, 1, 1, 0, 14, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(paneListeValeurEtFonctionsValUnitex, constraints);
    setConstraints(constraints, 0, 9, 1, 1, 0, 14, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label6, constraints);
    setConstraints(constraints, 1, 9, 2, 1, 30, 14, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(editeurUnitey, constraints);
    setConstraints(constraints, 1, 10, 1, 1, 0, 14, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(paneListeValeurEtFonctionsValUnitey, constraints);
   
    setConstraints(constraints, 3, 0, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(btnOK, constraints);
    setConstraints(constraints, 3, 1, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(btnCancel, constraints);
    // Si on cre un repre on propose de crer des axes si ncessaire
    if (!modification) {
	    setConstraints(constraints, 0, 11, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
	    pane.add(checkCreerAxes, constraints);
    }
    
    setContentPane(pane);
    getRootPane().setDefaultButton(btnOK);    
    pack();
  }

	private void btnOKActionPerformed() {
    // Si la rponse est valide, on ferme la boite de dialogue sans la dtruire.
    // VrifieValidite se charge de changer l'angle de la rotation si la rponse est
    // correcte.
    if (editeurOrigineAbscisses.verifieValidite()) {
    	if (editeurOrigineOrdonnees.verifieValidite()) {
    		if (editeurUnitex.verifieValidite()) {
    			if (editeurUnitey.verifieValidite()) {
		        setRetourOK(true);
		        // On abandonne les objets CValeurAssocieeAVariable de repere pour les remplacer par les locaux
		        repere.abscisseOrigine = valOrigineAbscisses;
		        repere.ordonneeOrigine = valOrigineOrdonnees;
		        repere.unite_x = valUnitex;
		        repere.unite_y = valUnitey;
		        // Modification version 4.7.4 pour plus de cohrence
		        // repere.quadrillageHorizontal = checkQuadHor.isSelected();
		        // repere.quadrillageVertical = checkQuadVer.isSelected();
		        repere.quadrillageHorizontal = checkQuadVer.isSelected();
		        repere.quadrillageVertical = checkQuadHor.isSelected();
		        
		        repere.pointilles = checkPointilles.isSelected();
		        repere.style = listeStyleTrait.styleTrait();
		        repere.couleur = colorButton.couleur;
		        if (modification) frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil78"));
		        dispose();
    			}
    		}
    	}
    	else return;
    }
  }
	
	private void btnCancelActionPerformed() {
		dispose();
	}

	
}
