/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package mathgraph32;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JDialog;

import objets.CAffichageLieAPoint;

/**
 *
 * @author Yves
 */
@SuppressWarnings("serial")
public class MtgColorButton extends JButton implements ActionListener, MouseListener {
  private MtgFrame frame;
  private static Insets insets = new Insets(0,0,0,0);
  public Color couleur = Color.BLACK; // la couleur du bouton
  // Dans les botes de dialogue d'entre d'un commentaire, quand on change la couleur de fond,
  // le commentaire associ doit tre mis  jour
  public CAffichageLieAPoint affichageAssocie;
  
  /** Creates a new instance of MtgColorButton */
  public MtgColorButton(MtgFrame frameInit) {
    super();
    frame = frameInit;
    setMargin(insets);
    setBackground(frame.paneRight.getBackground());
    addActionListener(this);
    addMouseListener(this);
    setPreferredSize(new Dimension(48,24));
    setToolTipText(MtgFrame.getString("choixCouleurDlg1"));
    affichageAssocie = null;
  }
  
  public void associeA(CAffichageLieAPoint affichageAssocieInit) {
  	affichageAssocie = affichageAssocieInit;
  }

  public void actionPerformed(ActionEvent e) {
    JColorChooser cs = frame.colorChooser;
    cs.setColor(couleur);
    JDialog dialog = JColorChooser.createDialog(frame, MtgFrame.getString("choixCouleurDlg1"), true, cs,
            null, null);
		dialog.setLocationRelativeTo(frame); // Ajout version 4.7.2
    dialog.setVisible(true);
    couleur = cs.getColor();
    frame.paneTransparence.donneCouleurBoutonTransparence(couleur);
    if (affichageAssocie != null) affichageAssocie.couleurFondAffichage = couleur;
  }
  
  @Override
  public void paint(Graphics g) {
  	Graphics2D g2D = (Graphics2D)g;
  	setSize(getPreferredSize());
  	setOpaque(false);
  	super.paint(g); // Pour que le rectangle de fond soit transparent
  	g2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    Dimension rect = getSize();
		g2D.setColor(couleur);
    g2D .fillOval(4, 4, rect.width - 9, rect.height - 9);
    // Supprim version 4.6
    // if (couleur == Color.BLACK) g2D.setColor(Color.BLACK);
    // else g2D.setColor(Color.WHITE);
    // g2D.drawOval(4, 4, rect.width - 9, rect.height - 10);
	}
  
  public void mouseClicked(MouseEvent e) {
  }

  public void mouseEntered(MouseEvent e) {
    if (getCursor() != MtgFrame.defaultCursor) setCursor(MtgFrame.defaultCursor);
  }

  public void mouseExited(MouseEvent e) {
  }

  public void mousePressed(MouseEvent e) {
  }

  public void mouseReleased(MouseEvent e) {
  }

  
}
