/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

// Modification de la version 2.5 : cette classe n'est plus abstract car elle peut tre
// instancie lors des chargements de prototypes
public class CCalculAncetre extends CValeurDynamique {
  public String nomCalcul;
  
  /** Creates new CCalculAncetre */
  public CCalculAncetre() {
  }

  public CCalculAncetre(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit) {
    super(liste, impProtoInit, estElementFinalInit);
  }
  
  // Constructeur pour clonage
  public CCalculAncetre(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    String nomCalculInit) {
    super(liste, impProtoInit, estElementFinalInit);
    // listeProprietaire = liste; Modifi version 2.5
    nomCalcul = new String(nomCalculInit);
  }
  
  @Override
  public String nom() {
    return nomCalcul;
  }
  
  @Override
  public void donneNom(String  nouveauNom) {
  	nomCalcul = nouveauNom;
  }
  
  /**
   * Fonction renvoyant pour une fonction le nombre de variables formelles
   * A redfinir pour CFonc, CFoncNVar, CSuiteRec et CSuiteRecComplexe
   */
  public int nombreVariables() {
  	return 0;
  }  
  
  double rendValeurFonction(boolean infoRandom, double parametre)
  throws ErreurCalculException {
    return rendValeur(infoRandom);
  }
  // Ajout pour la version 2.0
  // Appel pour les appels de fonctions. Le rsultat est renvoy dans zRes
  // Seuls les descendants sont appels
  void rendValeurFonctionComplexe(boolean infoRandom, Complexe parametre, Complexe zRes)
  throws ErreurCalculException {
  }
  
  /**
   * Ajout pour la version 3.0 : Renvoie true seulement si le calcul associ
   * est constant et ne fait pas appel  la fonction Rand()
   * @return
   */
  public boolean estConstant() {
  	return false;
  }
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    nomCalcul = inps.readCString();
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    nomCalcul = inps.readUTF();
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeUTF(nomCalcul);
  }  
}
