/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// Java : Important : en C++ CDemiDroite hritait de CDroite mais appelait la mthode de srialization
// de CElement ligne car il est inutile de mmoriser l'origine et le vecteur
// c'est impossible en java. Une demi-droite ne pouvant pas tre nomme, inutile s'avoir
// un champ abscisseNom
// Mais cela pose un problme car dans la mthode de serialization de CDemiDroiteOA qui hrite de
// CdemiDroite, c'est la mthode de srialization de CDroite qui tait appele, ce qui est illogique.
// Dans chargeDepuisFlux de CDemiDroiteOA, il faut donc dpiler les donnes inutiles
// Mais pourtant une demi-droite doit descendre de l'anctre des droites. J'ai rsolu le problme
// en rajoutant un type CDroiteAncetre qui soit l'anctre de toutes les droites
package objets;


import glob.*;

import java.awt.geom.*;
import java.awt.geom.Point2D.Double;
import java.awt.*;


import types.*;

// Modification pour la version 1.9.5
// A la suite du rajout des lieux d'objets cette classe n'est plus abstract
public class CDroiteAncetre extends CElementLigne {
  private Point2D.Double pointr = new Point2D.Double(); // Pour la rapidit
  public double xext1,yext1;	/// coordonnees d'un des points extremes
  /// affichs  l'cran
  public double xext2,yext2;	/// celles du deuxime
  public double point_x;
  public double point_y;
  public VecteurDirecteur vecteur;
  // Ajout versio 3.2.0
  boolean horsFenetre;
  
  /** Creates new CDroiteAncetre */
  public CDroiteAncetre() {
    vecteur = new VecteurDirecteur();
  }
  
  // Constructeur pour clonage
  public CDroiteAncetre(CListeObjets liste,CImplementationProto impProtoInit,boolean estElementFinalInit,
    Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit,
    boolean masqueInit, String nomInit, int indiceTailleNomInit, int styleInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
      dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit, styleInit);
    vecteur = new VecteurDirecteur(); // Pour que le vecteur directeur soit cr
  }
  // Constructeur pour les descendants de CdroiteAncetre n'ayant pas besoin d'tre nommes
  public CDroiteAncetre(CListeObjets liste, CImplementationProto impProtoInit,boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, int styleInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, false, 0, 0, masqueInit, "", 0, styleInit);
    vecteur = new VecteurDirecteur();
  }
  
  /** Constructeur pour les descendants de CdroiteAncetre nayant pas besoin d'tre nomms avec un nom ide  */
  public CDroiteAncetre(CListeObjets liste, CImplementationProto impProtoInit,boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, int styleInit, int indiceTailleNomInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, false, 0, 0, masqueInit, "", 0, styleInit);
    vecteur = new VecteurDirecteur();
  }
  
  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
    xext1 = ((CDroiteAncetre)ptel).xext1;
    yext1 = ((CDroiteAncetre)ptel).yext1;
    xext2 = ((CDroiteAncetre)ptel).xext2;
    yext2 = ((CDroiteAncetre)ptel).yext2;
    point_x = ((CDroiteAncetre)ptel).point_x;
    point_y = ((CDroiteAncetre)ptel).point_y;
    ((CDroiteAncetre)ptel).vecteur.setCopy(vecteur);
    // Ajout version 3.1.8
    horsFenetre = ((CDroiteAncetre)ptel).horsFenetre;
  }
  
  public boolean appartientA(double x,double y) {
    return existe;
  }
  
  // Ajout version 3.2.5
  /**
   * Fonction renvoyant la position relative de l'objet droite (ou un descendant) avec un autre objet
   * du type point, droite ou cercle.
   * L'entier retourn doit tre dfini par la classe PositionRelative
   */
  @Override
  public int positionRelative(CElementGraphique elg, Bool bperpendiculaires){
  	bperpendiculaires.setValue(false);
  	if (elg.estDeNature(NatureObjet.NToutPoint)) {
  		CPointAncetre pt = (CPointAncetre) elg;
  		VecteurDirecteur vec = new VecteurDirecteur(pt.x, pt.y, point_x, point_y);
  		if (Kernel.colineaires(vecteur, vec) && appartientA(pt.x, pt.y)) return PositionRelative.appartient;
  		else return PositionRelative.nappartient;
  	}
  	else {
  		if (elg.estDeNature(NatureObjet.NTouteDroite)) {
  			CDroiteAncetre dte = (CDroiteAncetre) elg;
  			// On regarde d'abord si les droites sont perpendiculaires
  			if (Kernel.zero(vecteur.x*dte.vecteur.x+vecteur.y*dte.vecteur.y)) bperpendiculaires.setValue(true);
  			Double pointInt = new Double();
				byte positionDroites = Kernel.intersection(point_x, point_y, vecteur, dte.point_x, dte.point_y, dte.vecteur, pointInt);
				switch (positionDroites) {
					case PositionDroites.Confondues :
						if ((getNature() == NatureObjet.NDroite) && (dte.getNature() == NatureObjet.NDroite))	return PositionRelative.confondus;
							else return PositionRelative.memeSupportDroite;							
					case PositionDroites.Secantes :
						if (appartientA(pointInt.x, pointInt.y) && dte.appartientA(pointInt.x, pointInt.y)) return PositionRelative.secants;
						else return PositionRelative.vide;
					case PositionDroites.Paralleles : return PositionRelative.paralleles;
					default : return PositionRelative.vide; // Pour satisfaire le compilateur
				}
  		}
  		else {
    		if (elg.estDeNature(NatureObjet.NToutCercle)) { // Cas d'un cercle ou un arc de cercle
    			CCercle cerc = (CCercle) elg;
    			Double point1 = new Double();
					Double point2 = new Double();
					byte positionDroiteCercle = Kernel.intersectionDroiteCercle(point_x, point_y, vecteur, cerc.centre_x, cerc.centre_y, cerc.rayon, point1, point2 );
    			switch (positionDroiteCercle) {
		    			case PositionDroiteCercle.Vide :
		    				return PositionRelative.vide;
		    			case PositionDroiteCercle.Secants : {
		    				boolean b1 = appartientA(point1.x, point1.y) && cerc.surArc(point1.x, point1.y);
		    				boolean b2 = appartientA(point2.x, point2.y) && cerc.surArc(point2.x, point2.y);
		    				if (b1 && b2) return PositionRelative.secants2Point;
		    				if (b1 || b2) return PositionRelative.secants1Point;
		    				return PositionRelative.vide;
		    			}
		    			case PositionDroiteCercle.Tangents : {
		     				boolean b1 = appartientA(point1.x, point1.y) && cerc.surArc(point1.x, point1.y);
		     				if (b1) return PositionRelative.tangents; else return PositionRelative.vide;
		    			}
		    			default : return PositionRelative.vide; // Pour satisfaire le compilateur
    			}
    		}
    		else return PositionRelative.vide; // Pour satisfaire le compilateur
  		}
  	}
  }
  
  // Compltement remani pour version 3.1.8
  // Revu version 4.7.7 car quand on avait une courbe si l'axe des abscisses disparaissait, la courbe disparaissait aussi.
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    double a,xa,ya,xb,yb,xc,yc,xd,yd;
    existe = vecteur.nonNul();
    if (!(existe)) return;
    horsFenetre = false;
    if (vecteur.x == 0) {
      xext1 = point_x;
      yext1 = 0;
      xext2 = point_x;
      yext2 = dimfen.longueurOrdonnees;
    	if ((point_x < 0) || (point_x > dimfen.longueurAbscisses)) {
    		horsFenetre = true;
    		return;
    	}
      positionneNom();
      return;
    };
    if (vecteur.y == 0) {
      xext1 = 0;
      yext1 = point_y;
      xext2 = dimfen.longueurAbscisses;
      yext2 = point_y;
    	if ((point_y < 0) || (point_y > dimfen.longueurOrdonnees)) {
    		horsFenetre = true;
    		return;
    	}
      positionneNom();
      return;
    }
    int indicePoint = 0;
    a = vecteur.y/vecteur.x;
    xa = 0;
    ya = point_y + a*(xa-point_x);
    if ((ya >=0) && (ya <= dimfen.longueurOrdonnees)) {
    	indicePoint++;
    	xext1 = xa;
    	yext1 = ya;
    }
    xb = dimfen.longueurAbscisses;
    yb = a*(xb-point_x)+point_y;
    if ((yb >= 0) && (yb <= dimfen.longueurOrdonnees)) {
    	indicePoint++;
    	if (indicePoint == 1) {
    		xext1 = xb;
    		yext1 = yb;
    	}
    	else {
    		xext2 = xb;
    		yext2 = yb;
        positionneNom();
    		return;
    	}
    }
    yc = 0;
    xc = (yc-point_y)/a+point_x;
    if ((xc > 0) && (xc < dimfen.longueurAbscisses)) {
    	indicePoint++;
    	if (indicePoint == 1) {
    		xext1 = xc;
    		yext1 = yc;
    	}
    	else {
    		xext2 = xc;
    		yext2 = yc;
        positionneNom();
    		return;
    	}
    }
    yd = dimfen.longueurOrdonnees;
    xd = (yd-point_y)/a+point_x;
    if ((xd > 0) && (xd <= dimfen.longueurAbscisses)) {
    	indicePoint++;
    	if (indicePoint == 2) {
    		xext2 = xd;
    		yext2 = yd;
        positionneNom();
    	}
    	else {
    		horsFenetre = true;
    	}  	
    }
    // Ligne suivante corrige version 4.1
    else horsFenetre = true;
    // Rajout version 4.7.7. On privilgie la direction horizontale.
    if (horsFenetre) {
    	xext1 = 0;
    	yext1 = ya;
    	xext2 = xb;
    	yext2 = yb;
    }
  }
  
  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
  	if (horsFenetre) return;
    int styleTrait;
    if (masque) styleTrait = StyleTrait.traitPointille;
    else styleTrait = style;
    // Certaines droites servent de faon interne (repres par ex ) et ne sont pas associs  une liste
    pDC.setStroke(Pinceau.CreePinceau(styleTrait, listeProprietaire));
    pDC.setColor(couleur);
    pDC.drawLine((int) Math.round(xext1),(int) Math.round(yext1),
      (int) Math.round(xext2),(int) Math.round(yext2));
  }  
  
  public double abscisseMaximale() {
    // Modification version 3.2.5.1 : Dans le cas d'une droite ou une demi-droite, si elle est hors-ecran,
  	// on renvoie la mme valeur que abscisseMinimale() ainsi un lieu de points ou un liue d'objets
  	// gnr par un point li  cette droite ou cette demi-droite n'existera plus.
  	// Pour les egments, ce n'est pas la mme chose et abscisseMinimale() et abscisseMaximale sont redfinies.
  	// if (horsFenetre) return abscisseMinimale(); // Modifi version 4.7.7 car si l'axe des abscisses pour une courbe disparaissait
  	// la courbe disparaissait aussi
    if ((vecteur.x == 0) && (vecteur.y == 0)) return 0;
    if (Math.abs(vecteur.x) >= Math.abs(vecteur.y))
      return (xext2-point_x)/vecteur.x;
    else
      return (yext2-point_y)/vecteur.y;
  }
  
  public double abscisseMinimale() {
    if ((vecteur.x == 0) && (vecteur.y == 0)) return 0;
    if (Math.abs(vecteur.x) >= Math.abs(vecteur.y))
      return (xext1-point_x)/vecteur.x;
    else
      return (yext1-point_y)/vecteur.y;
  }
  
  /**
   * Procdure utilise pour calculer la distance d'un point  un demi-plan
   * car on dsigne un demi-plan en dsignant la droite qui est
   * son bord et on peut dsigner le demi-plan mme si la droite est cache
   */
	double distancePointPourDemiPlan(double xp,double yp)
	{
		if (!existe) return -1;
		else
		{
			Kernel.projetteOrtho(xp,yp,point_x,point_y,vecteur,pointr);
			return Kernel.distancePointPoint(xp,yp,pointr.x,pointr.y);
		}  //if not(existe)
	}
  
  @Override
  public double distancePoint(double xp, double yp, boolean masquage) {
    double dis;
    if (!(existe) || (masquage && masque))
      return -1;
    else {
      Kernel.projetteOrtho(xp,yp,point_x,point_y,vecteur,pointr);
      dis = Kernel.distancePointPoint(xp,yp,pointr.x,pointr.y);
      return dis;
    }
  }
  
  /**
   * Fonction renvoyant true seulement si la droite est visible dans la fentre dimfen
   * @param dimfen
   */
  // Modifi version 3.2.4
  @Override
  public boolean dansFenetre(DimensionFenetre dimfen) {
    /*
  	Point2D.Double[] res = new Point2D.Double[4];
    for (int i = 0; i < 4; i++) res[i] = new Point2D.Double();

    Kernel.projetteOrtho(0,0,point_x,point_y,vecteur,res[0]);
    Kernel.projetteOrtho(dimfen.longueurAbscisses,0,point_x,point_y,vecteur,res[1]);
    Kernel.projetteOrtho(0,dimfen.longueurOrdonnees,point_x,point_y,vecteur,res[2]);
    Kernel.projetteOrtho(dimfen.longueurAbscisses,dimfen.longueurOrdonnees,point_x,point_y,vecteur,res[3]);
    boolean resul = false;
    // On regarde si au moins un des 4 projets est dans la toile d'affichage
    for (int i = 0; i < 4; i++)
      resul = resul || dimfen.dansFenetre(res[i].x,res[i].y);
    return resul;
    */
  	return !horsFenetre;
  }
  
  @Override
  public long getNature() {
    return NatureObjet.NDroite;
  }
  
  @Override
  public boolean coincideAvec(CElementBase p) {
    if (p.getNature() != NatureObjet.NDroite) return false;
    CDroiteAncetre dte = (CDroiteAncetre) p;
    VecteurDirecteur vec = new VecteurDirecteur(dte.point_x,dte.point_y, point_x, point_y);
    return (Kernel.colineaires(vecteur, dte.vecteur) && Kernel.colineaires(vec, this.vecteur));
  }
  /**
   * Fonction rajoute pour la version 2.1 et servant  savoir si un point
   * est par dfinition sur une droite. Renverra true par exemple
   * si la droite est dfinie par deux points et le point est un de ces deux points
   * ou si le point est li  la droite
   * A t rajout pour amliorer l'outil d'intersection
   * A redfinir pour les descendants quand ncessaire
   * */
  public boolean contientParDefinition(CPointAncetre po)
  {
    return false;
  }
  
  /**
   * Fonction rajoute par la version 1.2 pour savoir si une droite est prallle  une droite D
   * par dfinition d'aprs les rgles de la gomtrie : par exemple parallle par dfinition
   * ou image de D par une translation, symtrie centrale, homothtie
   * @return
   */
  public boolean estParalleleParDefinition(CDroiteAncetre dte){
  	// return (dte == this) || dte.estParalleleParDefinition(this);
  	return (dte == this);
  }
  
  public boolean estPerpendiculaireParDefinition(CDroiteAncetre dte) {
  	return false;
  }
  
  
	/**
	 * Fonction renvoyant true si pt1 et pt2 forment une droite parallle  this.
	 * Cela arrive quand les deux points sont des points images par une mme translation, homothtie
	 * ou symtrie centrale et quand leurs antcdents sont sur une droite parallle  this
	 * ou quand pt2 est l'image de pt1 par une translation de vecteur dont l'origine et l'extrmit
	 * sont tous les deux sur une droite parallel  D ou vice-versa
	 * @param pt1
	 * @param pt2
	 * @return
	 */
	public boolean formentDroiteParallele(CPointAncetre pt1, CPointAncetre pt2) {
		if (pt1.estPointImage() && pt2.estPointImage()) {
			CPointImage ptim1 = (CPointImage) pt1;
			CPointImage ptim2 = (CPointImage)pt2;
			if ((ptim1.transformation == ptim2.transformation) 
					&&((ptim1.transformation.natureTransformation() & (CTransformation.translation | CTransformation.homothetie | CTransformation.symetrieCentrale)) != 0))
				return listeProprietaire.sontParDefinitionSurParallele(ptim1.antecedent, ptim2.antecedent, this);
			else return false;
		}
		else {
			// On regarde d'abord si pt2 est l'image d'un point de this par une translation  dont l'origine est sur this
			// et l'extrmit est pt1 ou si pt2 est l'image de pt1 par une translation dont le vecteur a son origine et son extrmit
			// qui sont sur this
			if (pt2.estPointImage()) {
				CPointImage ptim = (CPointImage) pt2;
				if (ptim.transformation.natureTransformation() == CTransformation.translation) {
					CTranslation trans = (CTranslation) ptim.transformation;
					return ((trans.extremiteVecteur == pt1) && (trans.origineVecteur.appartientParDefinition(this) && ptim.antecedent.appartientParDefinition(this)))
							|| (ptim.antecedent == pt1) && listeProprietaire.sontParDefinitionSurParallele(trans.origineVecteur, trans.extremiteVecteur, this);
				}	
				else return false;
			}
			// On regarde ensuite si pt1 est l'image d'un point de this par une translation  dont l'origine est sur this
			// et l'extrmit est pt2 ou si pt1 est l'image de pt2 par une translation dont le vecteur a son origine et son extrmit
			// qui sont sur this
			if (pt1.estPointImage()) {
				CPointImage ptim = (CPointImage) pt1;
				if (ptim.transformation.natureTransformation() == CTransformation.translation) {
					CTranslation trans = (CTranslation) ptim.transformation;
					return ((trans.extremiteVecteur == pt2) && (trans.origineVecteur.appartientParDefinition(this) && ptim.antecedent.appartientParDefinition(this)))
							|| (ptim.antecedent == pt2) && listeProprietaire.sontParDefinitionSurParallele(trans.origineVecteur, trans.extremiteVecteur, this);
				}
				else return false;
			}
			else return false;
		}
	}
}

