/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.Color;
import java.awt.Graphics2D;
import java.io.IOException;

import javax.swing.JLabel;

import org.scilab.forge.jlatexmath.TeXConstants;
import org.scilab.forge.jlatexmath.TeXFormula;
import org.scilab.forge.jlatexmath.TeXIcon;

import dialogs.EditeurFormuleDlg;
import dialogs.MtgDlg;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import types.NatureCalcul;
import types.NatureObjet;
import mathgraph32.MtgFormulaEditor;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class CEditeurFormule extends CAffichageLieAPoint { 
	public CCalcul calculAssocie; // Le calcul que le champ d'dition sera charg de modifier
	public int nbcol; // Nombre de caractres permis pour l'dition
	public String enTete;
	public MtgFormulaEditor mtgFormulaEditor;
	public boolean dejaAffiche = false;
	// Modification version 4.6 : on rajoute un champ variables1Car qui sera true si le champ attend des calculs
	// o les variables n'ont qu'un seul caractre et o les signes de multiplication peuvent tre omis
	// D'o un numro de version de l'objet qui passe  2.
	public boolean variables1Car;
	// Ajout version 4.7.2. Possibilit d'afficher la formule en LaTeX  droite de l'diteur prcd d'une signe =
	public boolean affichageFormule;
	public String preCodeLaTex; // Le code LaTeX qui sera insr avant la formule dans le cas o on demande l'affichage de la formule
	public TeXFormula formula;
	private TeXIcon icon;
	private JLabel jl = new JLabel(); // Sert  afficher l'image de l'icne
	// Ajout version 4.7.2 pour savoir si la formule doit tre affiche ou non (dans le cas o on demande l'affichage de la formule
	// et que le calcul est 1/0  on n'affiche rien
	// private boolean isNotZeroInverse; // Supprim version 4.7.2.1 Remplac par un paramtre supplmentaire
	public boolean affLaTeXPremierAffichage;	// Nouvelle variable : Si true, et si affichageFormule est true, lors du premier affichage
																							// de la formule, la formule LaTeX n'est pas affiche.
	public boolean affichageTempsReel;
	public boolean premierAffichage; // true tant que l'utilisateur n'a pas entr une formule valide par entre.
	
  @Override
  public int numeroVersion() {
  	// Modification version 4.7.2 o on donne la possibilit d'afficher  droite du champ d'dition
  	// la formule en LaTeX correspondante
  	// return 2;
  	// Modifi versionn 4.7.2.1 o on donne la possibilit de ne pas afficher la formule au premier affichage 
  	// return 3;
  	return 4;
  }  

	/**
	 * Constructeur par dfaut pour serialisation
	 */
	public CEditeurFormule() {
		premierAffichage = true;
	}
	
  // Constructeur pour clonage
	// L'objet CEditeurFormule ne possde qu'un seul type d'alignement et e peut pas tre li  un point
  public CEditeurFormule(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit,double x_nomInit, double y_nomInit, double dec_xInit, double dec_yInit, boolean masqueInit,
      int indiceTaillePoliceInit, int encadrementInit,
      boolean effacementFondInit, Color couleurFondInit, CCalcul calculAssocieInit, String enTeteInit,
      int nbcolInit, boolean affichageFormuleInit, String preCodeLaTeXInit, boolean variables1CarInit,
      boolean affLaTeXPremierAffichageInit, boolean affichageTempsReelInit) {
        super (liste, impProtoInit, estElementFinalInit, couleurInit, x_nomInit, y_nomInit,
          dec_xInit, dec_yInit, masqueInit, null, indiceTaillePoliceInit,
          encadrementInit, effacementFondInit, couleurFondInit, CAffichageLieAPoint.alignementHorizontalDroit,
          CAffichageLieAPoint.alignementVerticalHaut);
        calculAssocie = calculAssocieInit;
        enTete = enTeteInit;
        nbcol = nbcolInit;
        mtgFormulaEditor = new MtgFormulaEditor(this);
        affichageFormule =affichageFormuleInit;
        preCodeLaTex = preCodeLaTeXInit;
        variables1Car = variables1CarInit;
        affLaTeXPremierAffichage = affLaTeXPremierAffichageInit;
        affichageTempsReel = affichageTempsReelInit;
        premierAffichage = true;
  }
        
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(calculAssocie);
    int ind3 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CEditeurFormule(listeCible, (CImplementationProto)listeCible.get(ind3, CImplementationProto.class),
      estElementFinal, couleur,x_nom, y_nom, dec_x, dec_y, masque, 
      indiceTaillePolice, encadrement, effacementFond, couleurFondAffichage,
      (CCalcul)listeCible.get(ind1, CCalcul.class), enTete, nbcol, affichageFormule, preCodeLaTex, variables1Car,
      affLaTeXPremierAffichage, affichageTempsReel);
    ((CEditeurFormule) ptelb).mtgFormulaEditor.setVisible(!masque);
    // Ajout version 4.7.2
    if (listeCible.getClass() != CPrototype.class) {
    	((CEditeurFormule) ptelb).creeFormule();
    }
    //
    return ptelb;
  }
    
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || calculAssocie.dependDe(p));
  }

  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    return super.dependDePourReclassement(p) || calculAssocie.dependDePourReclassement(p);
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return super.dependDePourBoucle(p) || calculAssocie.dependDePourBoucle(p);
  }
  
  /**
   * Pour cet objet, c'est au niveau de affiche que, lors du premier affichage, le MtgFormulaEditor
   * est rajoute au paneCenter du MtgInternalFrame
   */
  @Override
  public void affiche(Graphics2D pDC, boolean masquage, Color couleurFond,
  		float transparence) {
  	// Modifi version 4.7.0.
  	// Si on est en train de copier-coller ou d'exporter dans un fichier image,
  	// on n'affiche pas cet objet car un comoosant d'dition ne peut pas s'afficher.
  	if (listeProprietaire.bCreationComposants) {
  		super.affiche(pDC, masquage, couleurFond, transparence);
	  	if (!dejaAffiche && ! masque) {
	  		// mtgFormulaEditor = new MtgFormulaEditor(this);
		  	mtgFormulaEditor.setPositionOnInternaleFrame();
		  	dejaAffiche = true;
	  	}
	  	// Modification version 4.7.2
	  	// Si la formule contenue est 1/0, rien n'est affich
	  	// Modifi version 4.7.2.1
	  	// if (!masque && affichageFormule && (formula != null) && isNotZeroInverse) {
	  	if (!masque && affichageFormule && (formula != null) && !(premierAffichage && !affLaTeXPremierAffichage)) {
		  	int width = mtgFormulaEditor.getWidth();
		    icon.setForeground(couleur);
		    int xicon = rectangleAffichage.x + rectangleAffichage.width + width;
		    int yicon = rectangleAffichage.y + (int) (rectangleAffichage.height/2 - icon.getIconHeight()/2);
		    if (effacementFond) {
	        pDC.setColor(couleurFondAffichage);
		      pDC.fillRect(xicon, yicon, icon.getIconWidth(), icon.getIconHeight());
		    }
		   	icon.paintIcon(jl, pDC, xicon, yicon);
	  	}
  	}
  	else {
  		super.affiche(pDC, masquage, couleurFond, transparence);
	  	if (!masque && affichageFormule) {
		    icon.setForeground(couleur);
		    int xicon = rectangleAffichage.x + rectangleAffichage.width;
		    int yicon = rectangleAffichage.y + rectangleAffichage.height/2 - icon.getIconHeight()/2;
		    if (effacementFond) {
	        pDC.setColor(couleurFondAffichage);
		      pDC.fillRect(xicon, yicon, icon.getIconWidth(), icon.getIconHeight());
		    }
		   	icon.paintIcon(jl, pDC, xicon, yicon);
	  	} 		
  	}
  }
  
  @Override
  public String rendChaineAffichage(boolean infoRandom) {
  	if (enTete.isEmpty()) {
  		String ch = calculAssocie.nomCalcul;
  		if (calculAssocie.estDeNatureCalcul(NatureCalcul.NFonction | NatureCalcul.NFonctionComplexe)) {
  			CFonc fonc = (CFonc) calculAssocie;
  			ch = ch + CCalculBase.parouv + fonc.chaineVariable + CCalculBase.parfer;
  		}
  		else {
  			if (calculAssocie.estDeNatureCalcul(NatureCalcul.NTouteFonctionReelleOuComplexe)) {
	  			CFoncNVar fonc = (CFoncNVar) calculAssocie;
	  			ch = ch + CCalculBase.parouv;
	  	    for (int i = 0; i < fonc.nbVar; i++) {
	  	    	ch = ch + fonc.chaineVariable[i];
	  	    	if (i != fonc.nbVar - 1) ch = ch + ',';
	  	    }
	   			ch = ch + CCalculBase.parfer;
  			}
  		}
  		if (listeProprietaire.bCreationComposants || !affichageFormule) ch = ch + " = ";
  		else ch = ch + "\\;";
			// Modification version 4.7.2 : Affichage par dfaut en LaTeX de l'en-tte
  		ch = "$" + ch + "$";
  		return ch;
  	}
  	else return enTete;
  }

  @Override
  public long getNature() {
    return NatureObjet.NEditeurFormule;
  }

  @Override
  public void montre() {
  	super.montre();
  	if (mtgFormulaEditor != null) mtgFormulaEditor.setVisible(true);
  }
  
  @Override
  public void cache() {
  	super.cache();
  	if (mtgFormulaEditor != null) mtgFormulaEditor.setVisible(false);  	
  }
  
  @Override
  public void donneCouleur(Color coul) {
  	super.donneCouleur(coul);
  	mtgFormulaEditor.setForeground(coul);
  	// mtgFormulaEditor.setBackground(couleurFondAffichage); // Abandonn version 4.6
  }
  
  @Override
  public String infoPourHistorique() {
    String ch = getString("chinfo183") + " " + calculAssocie.nom();
    if (pointLie != null) ch = ch + "\n" + getString("chinfo49");
    return ch;
  }
  
  @Override
  public String chaineDesignation() {
    return getString("designationEditeurFormule");
  }
  
  @Override
  public boolean modifiableParMenu() {
  	return true; // A revoir aprs test
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	 return new EditeurFormuleDlg(frame, this, true);
  }

   @Override
   public String informationModification() {
   	return getString("chaineOutil155");
   }

   @Override
  public void placeNom(double xn, double yn) {
  	super.placeNom(xn, yn);
  	mtgFormulaEditor.setPositionOnInternaleFrame();
  }
   
  @Override
 	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
   	return
 		super.estDefiniParObjetDans(listeOb)
 		&&
 		(listeOb.contains(calculAssocie) || calculAssocie.estDefiniParObjetDans(listeOb));
 	}
  
  @Override
  public void translateDe(double decalagex, double decalagey) {
  	super.translateDe(decalagex, decalagey);
  	mtgFormulaEditor.setPositionOnInternaleFrame();
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)
  		throws IOException {
  	// TODO Auto-generated method stub
  	super.read(inps, list);
    int ind1 = inps.readInt();
    calculAssocie = (CCalcul) (list.get(ind1,CFonc.class));
  	enTete = inps.readUTF();
  	nbcol = inps.readInt();
  	if (nVersion < 2) variables1Car = false;
  	else variables1Car = inps.readBoolean();
  	// Modification version 4.7.2
  	if (nVersion < 3) {
  		affichageFormule = false;
  		preCodeLaTex = "";
  	}
  	else {
  		affichageFormule = inps.readBoolean();
  		preCodeLaTex = inps.readUTF();
  	}
  	if (nVersion < 4) {
  		affLaTeXPremierAffichage = true;
  		affichageTempsReel = false;
  	}
  	else {
  		affLaTeXPremierAffichage = inps.readBoolean();
  		affichageTempsReel = inps.readBoolean();
  	}
  	// Modification version 4.7.0.3 : On ne cre le composant que si la liste propritaire l'autorise via sa variable creationComposant
  	if (listeProprietaire.bCreationComposants) {
	  	mtgFormulaEditor = new MtgFormulaEditor(this);
	  	mtgFormulaEditor.setVisible(!masque);
	  	mtgFormulaEditor.setForeground(couleur);
  	}
  	// Ajout version 4.7.1
    if (list.getClass() != CPrototype.class) creeFormule();
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)
  		throws IOException {
  	// TODO Auto-generated method stub
  	super.write(oups, list);
    int ind1 = list.indexOf(calculAssocie);
    oups.writeInt(ind1);  	
  	oups.writeUTF(enTete);
  	oups.writeInt(nbcol);
  	oups.writeBoolean(variables1Car);
  	// Ajout version 4.7.2
  	oups.writeBoolean(affichageFormule);
  	oups.writeUTF(preCodeLaTex);
  	// Ajout version 4.7.2.1
  	oups.writeBoolean(affLaTeXPremierAffichage);
  	oups.writeBoolean(affichageTempsReel);
  }
  
  // Ajout version 4.5.2
  @Override
  public void retireComposant(MtgInternalFrame cadre) {
  	cadre.paneCenter.remove(mtgFormulaEditor);  
  }
  
  @Override
  public void montreComposant() {
  	mtgFormulaEditor.setVisible(true);
  }
  
  @Override
  public void cacheComposant() {
  	mtgFormulaEditor.setVisible(false);
  }

  // Ajout version 4.6
  @Override
  public void metAJour() {
  	if (calculAssocie.existe && !mtgFormulaEditor.getText().isEmpty()) mtgFormulaEditor.setText(calculAssocie.chaineCalcul);
  	else mtgFormulaEditor.setText("");
  	creeFormule(); // Pour rafficher la formule si elle est affiche
  }
  
  /**
   * Ajout version 4.7.2
   * Fonction crant la formule
   * A cause de l'applet de setExternalConverterFactory, ne doit pas tre appele
   * simultanment par plusieurs thread
   * Si formuleEnCreation est  true c'est qu'un autre Thread est en train d'utiliser CreeFormule
   * et on attend qu'il ait fini
   */
  public synchronized void creeFormule() {
  	/* Supprim version 4.7.4
  	if (CLatex.formuleEnCreation) {
  		while (CLatex.formuleEnCreation) Thread.yield();
  	}
  	CLatex.formuleEnCreation = true;
  	*/
  	// DynamicAtom.setExternalConverterFactory(this);
  	try {
      String[] varFor = null;
      if (calculAssocie.estDeNatureCalcul(NatureCalcul.NFonction | NatureCalcul.NFonctionComplexe)) {
      	CFonc f = (CFonc) calculAssocie;
      	varFor = new String[1];
      	varFor[0] = f.chaineVariable;
      }
      else {
        if (calculAssocie.estDeNatureCalcul(NatureCalcul.NFonction2Var | NatureCalcul.NFonction3Var
        		| NatureCalcul.NFonctionComplexe2Var | NatureCalcul.NFonctionComplexe3Var)) {
  	    	CFoncNVar fnv = (CFoncNVar) calculAssocie;
  	    	varFor = new String[fnv.nbVar];
  	    	for (int k = 0; k < fnv.nbVar; k++) varFor[k] = new String(fnv.chaineVariable[k]);
        }
      }
  		formula =  new TeXFormula(preCodeLaTex + calculAssocie.calcul.chaineLatexSansPar(varFor));
    	icon = formula.createTeXIcon(TeXConstants.STYLE_DISPLAY, (float)listeProprietaire.coefMult*CLatex.tailleLatex[indiceTaillePolice]);
  		// CLatex.formuleEnCreation = false; // Supprim version 4.7.4
    	// isNotZeroInverse = !calculAssocie.calcul.isZeroInverse(); // Supprim version 4.7.2.1
  	}
  	catch (Exception e) {
			formula = null;
			// CLatex.formuleEnCreation = false; // Supprim version 4.7.4
			// isNotZeroInverse = true; // Supprim version 4.7.2.1
		}
  }
  
  /**
   * Fonction ajoute version 4.7.2.1 qui sert  afficher la formule en temps rel
   * quand l'utilisateur entre une formule dans le champ d'dition
   * @param text
   */
  public synchronized void creeFormule(String text) {
  	/* Supprim version 4.7.4
  	if (CLatex.formuleEnCreation) {
  		while (CLatex.formuleEnCreation) Thread.yield();
  	}
  	CLatex.formuleEnCreation = true;
  	*/
  	// DynamicAtom.setExternalConverterFactory(this);
  	try {
  		formula =  new TeXFormula(preCodeLaTex + text);
    	icon = formula.createTeXIcon(TeXConstants.STYLE_DISPLAY, (float)listeProprietaire.coefMult*CLatex.tailleLatex[indiceTaillePolice]);
  		// CLatex.formuleEnCreation = false; // Supprim version 4.7.4
    	// isNotZeroInverse = !calculAssocie.calcul.isZeroInverse(); // Supprim version 4.7.2.1
  	}
  	catch (Exception e) {
			formula = null;
			// CLatex.formuleEnCreation = false; // Supprim version 4.7.4
			// isNotZeroInverse = true; // Supprim version 4.7.2.1
		}  	
  }

  @Override
  public void adaptePourHauteResolution(double coef) {
  	// Pour que la taille de l'icone soit en rapport avec le coefficient multiplicateur quand on fait un edition Copier.
  	super.adaptePourHauteResolution(coef);
  	creeFormule();
  }
}
