/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.geom.*;
import java.awt.Color;


import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;

import mathgraph32.MtgFrame;


public class CIntDroiteDroite extends CPointAncetre {
	private CDroiteAncetre droite1;
	private CDroiteAncetre droite2;
  private Point2D.Double pointInt = new Point2D.Double();

  /** Creates new CIntDroiteDroite */
  public CIntDroiteDroite() {
  }
  
  /* Constructeur pour clonage */
  public CIntDroiteDroite(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit,
      boolean masqueInit, String nomInit, int indiceTailleNomInit,
      int motifInit, CDroiteAncetre droite1Init, CDroiteAncetre droite2Init, boolean marquePourTraceInit) {
     super (liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit, 
       dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit, motifInit, marquePourTraceInit);
     droite1 = droite1Init;
     droite2 = droite2Init;
  }
        
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(droite1);
    int ind2 = listeSource.indexOf(droite2);
    int ind3 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CIntDroiteDroite(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom,
      indiceTailleNom, motif, (CDroiteAncetre)(listeCible.get(ind1, CDroiteAncetre.class)),
      (CDroiteAncetre)(listeCible.get(ind2, CDroiteAncetre.class)), marquePourTrace);
    return ptelb;
  }

  @Override
  public String infoPourHistorique() {
    return getString("chinfo15");
  }

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(droite1);
    liste.add(droite2);
  } 
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
  	return memoriseDependance(super.dependDe(p) || droite1.dependDe(p) || droite2.dependDe(p));
  }

  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	return super.dependDePourReclassement(p) || droite1.dependDePourReclassement(p) || droite2.dependDePourReclassement(p);
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
  	return ((p == this) || droite1.dependDePourBoucle(p) || droite2.dependDePourBoucle(p));
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {

    existe = droite1.existe && droite2.existe;
    if (!existe) return;
    byte pos = Kernel.intersection(droite1.point_x,droite1.point_y,droite1.vecteur,
      droite2.point_x,droite2.point_y,droite2.vecteur,pointInt);
    if (pos != PositionDroites.Secantes)
    {
      existe = false;
      return;
    }
    else
    {
      double x = pointInt.x;
      double y = pointInt.y;
      if (droite1.appartientA(x, y) && droite2.appartientA(x, y)) {
      	super.placeEn(x, y);
      	super.positionne(infoRandom, dimfen); // Ajout version 4.6.4
      }
      else existe = false;
    }
  }

  @Override
  public long getNature() {
    return NatureObjet.NPoint;
  }

  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    int ind2 = inps.readCInt();
    droite1 = (CDroiteAncetre) (list.get(ind1,CDroiteAncetre.class));
    droite2 = (CDroiteAncetre) (list.get(ind2,CDroiteAncetre.class));
  }
  
  @Override
  public boolean confonduAvec(CElementBase p) {
    CIntDroiteDroite pt;
    if (p.getClass() == this.getClass()) {
      pt = (CIntDroiteDroite) p;
      return (((droite1==pt.droite1) && (droite2==pt.droite2))
			|| ((droite1==pt.droite2) && (droite2==pt.droite1)));
    }
    else return false;
  }  

  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		(listeOb.contains(droite1) || droite1.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(droite2) || droite2.estDefiniParObjetDans(listeOb));
	}  	  
  
  /* Lecture depuis un flux java */  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    droite1 = (CDroiteAncetre) (list.get(ind1,CDroiteAncetre.class));
    droite2 = (CDroiteAncetre) (list.get(ind2,CDroiteAncetre.class));
  }  
  /* Enregistrement dans un flux java */
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(droite1);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(droite2);
    oups.writeInt(ind2);
  }  
  /** Fonction ajoute pour la version 2.1 en vue d'amliorer l'outil de cration d'intersection
   * Renvoie true si le point est sur la droite par dfinition
   * A redfinir pour les descendants concerns
   * */
  @Override
  public boolean appartientParDefinition(CDroiteAncetre droite)
  {
    return super.appartientParDefinition(droite) || (droite == droite1) || (droite == droite2);
  }
}
