/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.Color;

import dialogs.MacroDecrementationVariableDlg;
import dialogs.MtgDlg;

import types.*;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class CMacroDecrementationVariable extends CMacroIncrementationVariable {

  /** Creates new CMacroDecrementationVariable */
  public CMacroDecrementationVariable() {
    super();
  }

  // Constructeur pour clonage
  public CMacroDecrementationVariable(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, double x_nomInit, double y_nomInit, double dec_xInit, double dec_yInit, boolean masqueInit,
      CPointAncetre pointLieInit, int indiceTaillePoliceInit, int encadrementInit,
      boolean effacementFondInit, Color couleurFondInit, String intituleInit, String commentaireMacroInit,
      CVariableBornee variableAssocieeInit, int alignementHorizontalInit, int alignementVerticalInit) {
        super (liste, impProtoInit, estElementFinalInit, couleurInit, x_nomInit, y_nomInit,
          dec_xInit, dec_yInit, masqueInit, pointLieInit, indiceTaillePoliceInit, encadrementInit,
          effacementFondInit, couleurFondInit, intituleInit, commentaireMacroInit, variableAssocieeInit,
          alignementHorizontalInit, alignementVerticalInit);
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
   int ind1 = listeSource.indexOf(variableAssociee);
   int ind2 = listeSource.indexOf(pointLie);
   int ind3 = listeSource.indexOf(impProto);       
   CElementBase ptelb = new CMacroDecrementationVariable(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, x_nom, y_nom, dec_x, dec_y, masque,
      (CPointAncetre)listeCible.get(ind2, CPointAncetre.class), indiceTaillePolice, 
      encadrement, effacementFond, couleurFondAffichage, intitule, commentaireMacro,
      (CVariableBornee)(listeCible.get(ind1, CVariableBornee.class)), alignementHorizontal, alignementVertical);
    listeDep.initialise(listeCible.uniteAngle, listeCible.pointeurLongueurUnite);
    // On ne peut pas  ce niveau faire une image de la liste ListeDep dans la liste clone car tous
    // les lments n'ont pas encor t chargs.
    //((CMacroIncrementationVariable)ptelb).listeDep.setImage(listeDep, listeProprietaire,
    //  ptelb.listeProprietaire);
    return ptelb;
  }
  
  @Override
  public void actionDansPaint(MtgInternalFrame cadre) {
    boolean infor = cadre.infRand;
    DimensionFenetre dimfen = cadre.dimf;
    // Modifi pour la version 2.0
    // listeDep.effaceTout(cadre.gbuffer, dimfen, true);
    variableAssociee.decremente();
    listeDep.positionne(infor, dimfen);
    // Ligne suivante ajoute version 2.0
    cadre.effacegbuffer();
    cadre.afficheToutDansgbuffer();   
    executionEnCours = false;
  }
  
  @Override
  public boolean executionPossible() {
    boolean infr = false; // Pas de recalcul alatoire
    double nouvelleValeur = variableAssociee.rendValeur(infr) - variableAssociee.valeurPas;
    return (nouvelleValeur >= variableAssociee.valeurMini)
        && (nouvelleValeur <= variableAssociee.valeurMaxi);
  }
  
  @Override
  public String infoPourHistorique() {
    return getString("chinfo122") + " " + variableAssociee.nom();
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new MacroDecrementationVariableDlg(frame, this, true);
  }

}
