/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.Color;
import java.awt.Graphics2D;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public abstract class CMarqueAngleAncetre extends CCercle {
    // Les coordonnes du sommet de l'angle sont centre_x et centre_y, dfinis
    // dans l'objet anctre CCercle
    VecteurDirecteur u,v; // Vecteurs directeurs des cts de l'angle
    //intersections de l'angle droit avec les cts
    protected CPointBase point1 ; 
    protected CPointBase point2;
    //Sommet de l'angle droit
    protected CPointBase pointc;
    // Les ventuels segments de l'angle droit
    protected CSegment segment1, segment2;
    // L'ventuelle marque de segment de l'angle droit
    protected CMarqueSegment marqueSegment;
   
    // Les segments de l'angle droit
    protected double ang1rad,ang2rad; //angles polaires en radian
    // Ajout pour la version 2.5.2 : un int reprsentant le style de marque d'angle
    public int styleMarque;

  // Pour la version 2.5.2 on passe aux marques d'angle avec style de marque d'angle. Donc nouveau numro de version 2
  @Override
  public int numeroVersion() {
  	return 2;
  }
    
  /** Creates new CMarqueAngleAncetre */
  public CMarqueAngleAncetre () {
    u = new VecteurDirecteur();
    v = new VecteurDirecteur();
  }
  
  // Constructeur pour clonage.
  public CMarqueAngleAncetre(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, int styleInit, int styleMarqueInit, double rayonInit) {
    super (liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit);
    styleMarque = styleMarqueInit;
    rayon = rayonInit;
    u = new VecteurDirecteur();
    v = new VecteurDirecteur();
    point1 = new CPointBase();
    point2 = new CPointBase();
    pointc = new CPointBase();
    segment1 = new CSegment(liste, point2, pointc);
    marqueSegment = new CMarqueSegment(liste, null, false, couleurInit, false, styleInit,
    		StyleMarqueAngle.styleMarqueSegment(styleMarqueInit), segment1);
  }  

  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
    ((CMarqueAngleAncetre) ptel).u.setCopy(u);
    ((CMarqueAngleAncetre) ptel).v.setCopy(v);
    point1.placeEn(((CMarqueAngleAncetre)ptel).point1);
    point2.placeEn(((CMarqueAngleAncetre)ptel).point2);
    pointc.placeEn(((CMarqueAngleAncetre)ptel).pointc);
    ang1rad = ((CMarqueAngleAncetre)ptel).ang1rad;
    ang2rad = ((CMarqueAngleAncetre)ptel).ang2rad;
  }

  @Override
  public long getNature() {
    return NatureObjet.NMarqueAngle;
  }
  
  public void donneStyleMarque(int newStyleMarque) {
  	styleMarque = newStyleMarque;
  	marqueSegment.donneMotif(StyleMarqueAngle.styleMarqueSegment(newStyleMarque));
  }
  
  @Override
  public String chaineDesignation() {
    return getString("designationMarqueAngle");
  }
    
  @Override
  public void adaptePourHauteResolution(double coef) {
  	super.adaptePourHauteResolution(coef);
		rayon *= coef;
  }
  
  /*
   * A redfinir pour les marques d'angle orient
   */
  public boolean estOriente() {
  	return false;
  }
  
  public final void prepareAngleDroit()
	{
		double normeu,normev;
		VecteurDirecteur u1 = new VecteurDirecteur();
		VecteurDirecteur v1 = new VecteurDirecteur();
		normeu = u.norme();
		normev = v.norme();
		u1.x = u.x/normeu*rayon;
		u1.y = u.y/normeu*rayon;
		v1.x = v.x/normev*rayon;
		v1.y = v.y/normev*rayon;
		point1.x = centre_x + v1.x;
		point1.y = centre_y + v1.y;
		pointc.x = point1.x + u1.x;
		pointc.y = point1.y + u1.y;
		point2.x = centre_x + u1.x;
		point2.y = centre_y + u1.y;
	}  
  
  protected void traceAngleDroit(Graphics2D pDC)
    {
      pDC.drawLine((int)Math.round(point1.x),(int)Math.round(point1.y),
        (int)Math.round(pointc.x),(int)Math.round(pointc.y));
      pDC.drawLine((int)Math.round(point2.x),(int)Math.round(point2.y),
        (int)Math.round(pointc.x),(int)Math.round(pointc.y));
    }
  
  protected void traceMarqueAngle(Graphics2D pDC, double xdeb, double ydeb, double xcentre, double ycentre,
  		double ang) {
  	int x1, y1, x2, y2;
  	double xmilieu, ymilieu, xc1, yc1;
  	double ang2, ang3;
    // if (Kernel.zero(Math.PI - ang)) ang = Math.PI;
  	VecteurDirecteur vec1 = new VecteurDirecteur(xcentre, ycentre, xdeb, ydeb);
  	VecteurDirecteur vec = new VecteurDirecteur();
		VecteurDirecteur vect2 = new VecteurDirecteur();
  	vec1.tourne(ang/2, vec);
  	double nvec = vec.norme();
  	vec.x = vec.x/nvec;
  	vec.y = vec.y/nvec;
    double demilong;
    // Correction bug version 4.7.2
  	// if (basseDefinition) demilong = 5; else demilong = 50;
    if (listeProprietaire.basseDefinition) demilong = 5; else demilong = listeProprietaire.coefMult*5;
  	switch (styleMarque) {
  	case StyleMarqueAngle.marqueSimple1Trait :
		case StyleMarqueAngle.marquePleine1Trait :
	  	xmilieu = centre_x + vec.x*rayon;
	  	ymilieu = centre_y + vec.y*rayon;
			x1 = (int) (xmilieu - vec.x*demilong);
			y1 = (int) (ymilieu - vec.y*demilong);
			x2 = (int) (xmilieu + vec.x*demilong);
			y2 = (int) (ymilieu + vec.y*demilong);
			pDC.drawLine(x1, y1, x2, y2);
			break;
			
  	case StyleMarqueAngle.marqueSimple2Traits :
		case StyleMarqueAngle.marquePleine2Traits :
			ang2 = ang/6;
			ang3 = Math.PI/30;
			if (Math.abs(ang2) >= Math.abs(ang3)) ang2 = ang3;
			vec.tourne(ang2, vect2);
	  	xc1 = centre_x + vect2.x*rayon;
	  	yc1 = centre_y + vect2.y*rayon;			
			x1 = (int) (xc1 - vec.x*demilong);
			y1 = (int) (yc1 - vec.y*demilong);
			x2 = (int) (xc1 + vec.x*demilong);
			y2 = (int) (yc1 + vec.y*demilong);
			pDC.drawLine(x1, y1, x2, y2);
			vec.tourne(- ang2, vect2);
	  	xc1 = centre_x + vect2.x*rayon;
	  	yc1 = centre_y + vect2.y*rayon;			
			x1 = (int) (xc1 - vec.x*demilong);
			y1 = (int) (yc1 - vec.y*demilong);
			x2 = (int) (xc1 + vec.x*demilong);
			y2 = (int) (yc1 + vec.y*demilong);
			pDC.drawLine(x1, y1, x2, y2);
			break;

  	case StyleMarqueAngle.marqueSimple3Traits :
		case StyleMarqueAngle.marquePleine3Traits :
	  	xmilieu = centre_x + vec.x*rayon;
	  	ymilieu = centre_y + vec.y*rayon;
			x1 = (int) (xmilieu - vec.x*demilong);
			y1 = (int) (ymilieu - vec.y*demilong);
			x2 = (int) (xmilieu + vec.x*demilong);
			y2 = (int) (ymilieu + vec.y*demilong);
			pDC.drawLine(x1, y1, x2, y2);
			ang2 = ang/4;
			ang3 = Math.PI/20;
			if (Math.abs(ang2) >= Math.abs(ang3)) ang2 = ang3;
			vec.tourne(ang2, vect2);
	  	xc1 = centre_x + vect2.x*rayon;
	  	yc1 = centre_y + vect2.y*rayon;			
			x1 = (int) (xc1 - vec.x*demilong);
			y1 = (int) (yc1 - vec.y*demilong);
			x2 = (int) (xc1 + vec.x*demilong);
			y2 = (int) (yc1 + vec.y*demilong);
			pDC.drawLine(x1, y1, x2, y2);
			vec.tourne(- ang2, vect2);
	  	xc1 = centre_x + vect2.x*rayon;
	  	yc1 = centre_y + vect2.y*rayon;			
			x1 = (int) (xc1 - vec.x*demilong);
			y1 = (int) (yc1 - vec.y*demilong);
			x2 = (int) (xc1 + vec.x*demilong);
			y2 = (int) (yc1 + vec.y*demilong);
			pDC.drawLine(x1, y1, x2, y2);
			break;
  	case StyleMarqueAngle.marqueSimpleCroix :
		case StyleMarqueAngle.marquePleineCroix :
	  	xmilieu = centre_x + vec.x*rayon;
	  	ymilieu = centre_y + vec.y*rayon;
	  	vec.tourne(Math.PI/4, vect2);
			x1 = (int) (xmilieu - vect2.x*demilong);
			y1 = (int) (ymilieu - vect2.y*demilong);
			x2 = (int) (xmilieu + vect2.x*demilong);
			y2 = (int) (ymilieu + vect2.y*demilong);
			pDC.drawLine(x1, y1, x2, y2);
	  	vec.tourne(-Math.PI/4, vect2);
			x1 = (int) (xmilieu - vect2.x*demilong);
			y1 = (int) (ymilieu - vect2.y*demilong);
			x2 = (int) (xmilieu + vect2.x*demilong);
			y2 = (int) (ymilieu + vect2.y*demilong);
			pDC.drawLine(x1, y1, x2, y2);
		default:
			break;
		}
  	
  }
  
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    rayon = inps.readCDouble()/10;
    point1 = new CPointBase();
    point2 = new CPointBase();
    pointc = new CPointBase();
    segment1 = new CSegment(listeProprietaire, point2, pointc);
    marqueSegment = new CMarqueSegment(listeProprietaire, null, false, Color.black, false, StyleTrait.traitFin, StyleMarqueAngle.marqueSimple, segment1);
  }      

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    if (nVersion == 1) styleMarque = 0;
    else styleMarque = inps.readInt();
    rayon = inps.readDouble();
    point1 = new CPointBase();
    point2 = new CPointBase();
    pointc = new CPointBase();
    segment1 = new CSegment(listeProprietaire, point2, pointc);
    marqueSegment = new CMarqueSegment(listeProprietaire, null, false, Color.black, false, StyleTrait.traitFin, StyleMarqueAngle.marqueSimple, segment1);
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeInt(styleMarque);
    oups.writeDouble(rayon);
  }
}
