/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;


// Type de point d'intersection de deux objets
import java.io.*;
import java.awt.Color;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

import mathgraph32.MtgFrame;

public class CPointLieBipoint extends CPointAncetre {
	public CBipoint ptBipoint;
	public int indiceDansBipoint; // Indice du point associe dans le bipoint

  /** Creates new CPointLieBipoint */
  public CPointLieBipoint() {
  }
  
  // Constructeur pour clonage
  public CPointLieBipoint(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit,
      boolean masqueInit, String nomInit, int indiceTailleNomInit,
      int motifInit, CBipoint ptBipointInit, int indiceDansBipointInit, boolean marquePourTraceInit) {
     super (liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
       dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit, motifInit, marquePourTraceInit);
     ptBipoint = ptBipointInit;
     indiceDansBipoint = indiceDansBipointInit;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(ptBipoint);
    int ind2 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CPointLieBipoint(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom, indiceTailleNom, motif,
        (CBipoint)(listeCible.get(ind1, CBipoint.class)), indiceDansBipoint, marquePourTrace);
    return ptelb; 
  }

  @Override
  public String infoPourHistorique() {
    return ptBipoint.infoPourHistorique();
  }

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    ptBipoint.ajouteAntecedents(frame, liste);
  } 
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || ptBipoint.dependDe(p)); 
  }

  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    return super.dependDePourReclassement(p) || ptBipoint.dependDePourReclassement(p); 
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || ptBipoint.dependDePourBoucle(p)); 
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    CPointDansBipoint ptp;

    switch (indiceDansBipoint)
    {
      case 1:
        ptp = ptBipoint.point1;
        break;
      case 2:
        ptp = ptBipoint.point2;
        break;
      // Inutile mais exig par le compilateur java
      default : ptp = ptBipoint.point1;
    }
    existe = ptp.existe;
    if (!existe) return;
    super.placeEn(ptp.x, ptp.y);
    super.positionne(infoRandom, dimfen); // Ajout version 4.6.4
  }
      
  @Override
  public CElementBase antecedentDirect() {
    return ptBipoint;
  }
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    indiceDansBipoint = inps.readByte();
    int ind1 = inps.readCInt();
    ptBipoint = (CBipoint) (list.get(ind1,CBipoint.class));   
  }
  
  @Override
  public void read( MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    indiceDansBipoint = inps.readByte();
    int ind1 = inps.readInt();
    ptBipoint = (CBipoint) (list.get(ind1,CBipoint.class));   
  }
  
  @Override
  public void write( MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeByte(indiceDansBipoint);
    int ind1 = list.indexOf(ptBipoint);
    oups.writeInt(ind1);
  }  
  /** Fonction ajoute pour la version 2.1 en vue d'amliorer l'outil de cration d'intersection
   * Renvoie true si le point est sur la droite par dfinition
   * A redfinir pour les descendants concerns
   * */
  @Override
  public boolean appartientParDefinition(CDroiteAncetre droite)
  {
    return super.appartientParDefinition(droite) || ptBipoint.estDefiniPar(droite);
  }
  /** Fonction ajoute pour la version 2.1 en vue d'amliorer l'outil de cration d'intersection
   * Renvoie true si le point est sur la droite par dfinition
   * A redfinir pour les descendants concerns
   * */
  @Override
  public boolean appartientParDefinition(CCercle cercle)
  {
    return super.appartientParDefinition(cercle) || ptBipoint.estDefiniPar(cercle);
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return ptBipoint.estDefiniParObjetDans(listeOb);
	}
  
}
