/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.geom.*;
import java.awt.Color;


import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;

import mathgraph32.MtgFrame;


public class CPointProjete extends CPointAncetre {
	CPointAncetre A;
	CDroiteAncetre D;

  private Point2D.Double point = new Point2D.Double();
  
  /** Creates new CPointProjete */
  public CPointProjete() {
  }
  
  /** Constructeur pour clonage */
  public CPointProjete(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit, boolean masqueInit, String nomInit, int indiceTailleNomInit,
      int motifInit , CPointAncetre AInit, CDroiteAncetre DInit, boolean marquePourTraceInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit, dec_xInit, dec_yInit,
      masqueInit, nomInit, indiceTailleNomInit, motifInit, marquePourTraceInit);
    A = AInit;
    D = DInit;
  }
  /** Constructeur plus court pour objets visuels */
  public CPointProjete(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean masqueInit, int motifInit, CPointAncetre AInit, CDroiteAncetre DInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, false, 0d, 0d, masqueInit, "", 2, motifInit, false);
    A = AInit;
    D = DInit;
  }
  
  @Override  
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(A);
    int ind2 = listeSource.indexOf(D);
    int ind3 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CPointProjete(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom, indiceTailleNom, motif, 
      (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)), (CDroiteAncetre)(listeCible.get(ind2, CDroiteAncetre.class)),
      marquePourTrace);
    return ptelb;
  }  
  
  /**
   * Fonction renvoyant la chane  afficher dans la bote de dialogue d'historique
   * pour dcrire l'objet.
   * @return
   */
  @Override
  public String infoPourHistorique() {
    return getString("chinfo70");
  }
  
  /**
   * Fonction ajoutant  list les lments qui ont gnr l'objet.
   * @param liste
   */
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(A);
    liste.add(D);
  }  
  
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || A.dependDe(p) || D.dependDe(p));
  }

  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    return super.dependDePourReclassement(p) || A.dependDePourReclassement(p) || D.dependDePourReclassement(p);
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || A.dependDePourBoucle(p) || D.dependDePourBoucle(p));
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
		existe = A.existe && D.existe;
		if (existe)
		{
			Kernel.projetteOrtho(A.x,A.y,D.point_x,D.point_y,D.vecteur,point);
      double xp = point.x;
      double yp = point.y;
			existe = existe && D.appartientA(xp,yp);
			if (existe) {
				super.placeEn(xp,yp);
				super.positionne(infoRandom, dimfen); // Ajout version 4.6.4
			}
		}
  }

  public boolean confonduAvec(CElementBase p) {
    CPointProjete pt;
    if (p.getClass() == this.getClass()) {
      pt = (CPointProjete) p;
      return (A == pt.A) && (D == pt.D);
    }
    else return false;
  }
  
  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (A == ancienPoint) A = nouveauPoint;
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		(listeOb.contains(A) || A.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(D) || D.estDefiniParObjetDans(listeOb));
	}      
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
     super.chargeDepuisFlux(inps, list);
     int ind1 = inps.readCInt();
     int ind2 = inps.readCInt();
     A = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
     D = (CDroiteAncetre) (list.get(ind2,CDroiteAncetre.class));
  }
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
     int ind1 = inps.readInt();
     int ind2 = inps.readInt();
     A = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
     D = (CDroiteAncetre) (list.get(ind2,CDroiteAncetre.class));
  }
  
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(A);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(D);
    oups.writeInt(ind2);
  }
  /** Fonction ajoute pour la version 2.1 en vue d'amliorer l'outil de cration d'intersection
   * Renvoie true si le point est sur la droite par dfinition
   * A redfinir pour les descendants concerns
   * */
  public boolean appartientParDefinition(CDroiteAncetre droite)
  {
    return super.appartientParDefinition(droite) || (droite == D);
  }
}
