/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;
import glob.Kernel;
import java.awt.Color;
import java.util.ListIterator;
import java.util.Vector;
import types.Bool;
import types.Doub;
import types.Int;
import types.NatureObjet;
import types.VecteurDirecteur;

public class CPolygone extends CLignePolygonaleAncetre {

  /** Creates new CPolygone */
  public CPolygone() {
  	super();
  }
  
  public CPolygone(CListeObjets liste, Color couleurInit, int styleInit) {
  	super(liste, couleurInit, styleInit);
  }

  /** Constructeur pour clonage */
  public CPolygone(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, int styleInit, Vector<CNoeudPointeurSurPoint> collectionPointsInit) {
    super (liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit, collectionPointsInit);
  }
  
  /** Construction  partir d'un vecteur de CNouedPointeurSurPoint */
  public CPolygone(CListeObjets liste, Vector<CNoeudPointeurSurPoint> vecteur, Color couleurInit, int styleInit) {
  	super(liste, vecteur, couleurInit, styleInit);
  }
  

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    Vector<CNoeudPointeurSurPoint> colClone = new Vector<CNoeudPointeurSurPoint>();
    ListIterator<CNoeudPointeurSurPoint> li = collectionPoints.listIterator();
    while(li.hasNext()) {
      CNoeudPointeurSurPoint noeud = li.next();
      CNoeudPointeurSurPoint noeudClone = noeud.getClone(listeSource, listeCible);
      colClone.add(noeudClone);
    }
    int ind1 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CPolygone(listeCible, (CImplementationProto)(listeCible.get(ind1, CImplementationProto.class)),
      estElementFinal, couleur, masque, style, colClone);
    return ptelb;    
  }

  @Override
  public boolean confonduAvec(CElementBase p) {
    CPolygone pt;
  	CPointAncetre ptp1;
  	CPointAncetre ptp2;
  	int i,j;
  	boolean resultat;
  	
		if (p.getClass() == this.getClass()) {
		  pt = (CPolygone) p;
			int nombrePoints = collectionPoints.size();
			if (nombrePoints != pt.collectionPoints.size()) return false;
			// On regarde si un polynme a dj t cr avec les mmes sommets 
			// dans le	mme ordre. On ne regarde pas le dernier sommet qui
			// est le mme que le premier}
			nombrePoints--;
			j = 0;
			do
			{
				ptp1 = collectionPoints.get(0).pointeurSurPoint;
				ptp2 = pt.collectionPoints.get(j % nombrePoints).pointeurSurPoint;
				resultat = (ptp1 == ptp2);
				i = 1;
				while (resultat && (i <= nombrePoints-1))
				{
					ptp1 = collectionPoints.get(i).pointeurSurPoint;
					ptp2 = pt.collectionPoints.get((i+j) % nombrePoints).pointeurSurPoint;
					resultat = resultat && (ptp1 == ptp2);
					i++;
				};
				j++;
			}
			while ((j != nombrePoints) && !resultat);
			
			if (resultat) return true;
			// On regarde en tournant dans l'autre sens
			j = 0;
			do
			{
				ptp1 = collectionPoints.get(nombrePoints-1).pointeurSurPoint;
				ptp2 = pt.collectionPoints.get(j % nombrePoints).pointeurSurPoint;
				resultat = (ptp1 == ptp2);
				i = 1;
				while (resultat && (i <= nombrePoints-1))
				{
					ptp1 = collectionPoints.get(nombrePoints-1-i).pointeurSurPoint;
					ptp2 = pt.collectionPoints.get((i+j) % nombrePoints).pointeurSurPoint;
					resultat = resultat && (ptp1 == ptp2);
					i++;
				};
				j++;
			}
			while ((j != nombrePoints) && !resultat);
			return resultat;		  
		}
    else return false;
  }
  
  @Override
  public boolean coincideAvec(CElementBase p) {
    CPolygone pt;
  	CPointAncetre ptp1;
  	CPointAncetre ptp2;
  	int i,j;
  	boolean resultat;
    if (p.getNature() != NatureObjet.NPolygone) return false;
	  pt = (CPolygone) p;
		int nombrePoints = collectionPoints.size();
		if (nombrePoints != pt.collectionPoints.size()) return false;
		// On regarde si un polynme a dj t cr avec les mmes sommets 
		// dans le	mme ordre. On ne regarde pas le dernier sommet qui
		// est le mme que le premier}
		nombrePoints--;
		j = 0;
		do
		{
			ptp1 = collectionPoints.get(0).pointeurSurPoint;
			ptp2 = pt.collectionPoints.get(j % nombrePoints).pointeurSurPoint;
			resultat = (ptp1 == ptp2);
			i = 1;
			while (resultat && (i <= nombrePoints-1))
			{
				ptp1 = collectionPoints.get(i).pointeurSurPoint;
				ptp2 = pt.collectionPoints.get((i+j) % nombrePoints).pointeurSurPoint;
				resultat = resultat && ptp1.coincideAvec(ptp2);
				i++;
			};
			j++;
		}
		while ((j != nombrePoints) && !resultat);
		
		if (resultat) return true;
		// On regarde en tournant dans l'autre sens
		j = 0;
		do
		{
			ptp1 = collectionPoints.get(nombrePoints-1).pointeurSurPoint;
			ptp2 = pt.collectionPoints.get(j % nombrePoints).pointeurSurPoint;
			resultat = ptp1.coincideAvec(ptp2);
			i = 1;
			while (resultat && (i <= nombrePoints-1))
			{
				ptp1 = collectionPoints.get(nombrePoints-1-i).pointeurSurPoint;
				ptp2 = pt.collectionPoints.get((i+j) % nombrePoints).pointeurSurPoint;
				resultat = resultat && ptp1.coincideAvec(ptp2);
				i++;
			};
			j++;
		}
		while ((j != nombrePoints) && !resultat);
		return resultat;		      
  }  
  
  
  @Override
  public String infoPourHistorique() {
    return getString("chinfo59");
  }
  
  public double abscisseMaximale() {
    return 1-1e-12;
  }

  public double abscisseMinimale() {
    return 0;
  }

  @Override
  public long getNature() {
    return NatureObjet.NPolygone;
  }

  @Override
  public String complementInfo() {
    return getString("chinfo45");
  }
  
  // Rajout pour la version 1.9.5
	boolean lieALigneFermee() {
    return true;
  }
	
  public String nomSommetsPolygone() {
  	int lon = 0;
  	for (int i=0; i < collectionPoints.size() - 1; i++) {
      CNoeudPointeurSurPoint noeud = collectionPoints.get(i);
      CPointAncetre ptp = noeud.pointeurSurPoint;
      lon += ptp.nom.length();
    }
  	StringBuffer stb = new StringBuffer(lon);
  	for (int i=0; i < collectionPoints.size() - 1; i++) {
      CNoeudPointeurSurPoint noeud = collectionPoints.get(i);
	    CPointAncetre ptp = noeud.pointeurSurPoint;
	    stb.append(ptp.nom);
    }
    return stb.toString();
  }
	
  @Override
  public String chaineDesignation() {
    return getString("designationPolygone");
  }
	
  // Ajout version 2.7
  /**
   * Fonction renvoyant true si le point de coordonnes (x; y) est intrieur au polygne
   * c'est  dire s'il est sur un des cts ou si la somme des mesures principales d'angle orient
   * est un multiple impair de 2*pi.
   */
  public boolean pointInterieur(double x, double y) {
  	double sommeang = 0;
  	int taille = collectionPoints.size();
  	for (int i = 0; i < taille - 1; i++) {
  		CNoeudPointeurSurPoint noeud1 = collectionPoints.get(i);
  		double x1 = noeud1.pointeurSurPoint.x;
  		double y1 = noeud1.pointeurSurPoint.y;
  		CNoeudPointeurSurPoint noeud2 = collectionPoints.get(i + 1);
  		double x2 = noeud2.pointeurSurPoint.x;
  		double y2 = noeud2.pointeurSurPoint.y;
  		if (Kernel.appartientSegment(x, y, x1, y1, x2, y2)) return true;
  		VecteurDirecteur u = new VecteurDirecteur(x, y, x1, y1);
  		VecteurDirecteur v = new VecteurDirecteur(x, y, x2, y2);
  		sommeang += u.mesureAngleVecteurs(v);
  	}
  	double q = Math.floor(sommeang/(2*Math.PI)+0.5);
  	return q != 2*Math.floor(q/2);
  }
  
  
  /**
   * Fonction renvoyant la position d'un point intrieur  un polygone.
   * Supposons par exemple que le polygone soit ABCDE
   * @param x : l'abscisse du point intrieur
   * @param y : l'ordonne du point intrieur
   * @param indicePremierSommet
   * Pour que le point soit intrieur il doit tre intrieur  deux triangles conscutifs
   * en suivant les sommets dans l'ordre
   * par exemple BCD et DEF alors on calculera le point d'intersection de BCD et CDE on calculera le point I
 	 * d'intersection de BC et DE, alpha se rfre  I, beta  C et gamma  D.
   * @param alpha
   * @param beta
   * @param gamma
   * @param estSurCote : true si le point est sur un des cts. indicePremierSommet contient
   * alors l'indice du premier sommet de ce ct
   * @return : renvoie true si le point a bien t trouv comme intrieur
   */
  
  public boolean determinePositionInterieure(double x, double y, Int indicePremierSommet,
  		Bool estSurCote, Doub alpha, Doub beta, Doub gamma) {
  	int nbSommets = collectionPoints.size() - 1;
  	for (int i = 0; i < nbSommets; i++) {
  		CNoeudPointeurSurPoint nd1 = (CNoeudPointeurSurPoint) collectionPoints.get(i);
  		CPointAncetre B = nd1.pointeurSurPoint;
  		CNoeudPointeurSurPoint nd2 = (CNoeudPointeurSurPoint) collectionPoints.get(i+1);
  		CPointAncetre C = nd2.pointeurSurPoint;
  		if (Kernel.appartientSegment(x, y, B.x, B.y, C.x, C.y)) {
  			indicePremierSommet.setValue(i);
  			Kernel.coefficientsPointSurSegment(x, y, B.x, B.y, C.x, C.y, alpha, beta);
  			estSurCote.setValue(true);
  			return true;
  		}
  	}
  	CNoeudPointeurSurPoint nda = (CNoeudPointeurSurPoint) collectionPoints.get(0);
  	CPointAncetre A = nda.pointeurSurPoint;
  	for (int i = 1; i < nbSommets - 1; i++) {
  		CNoeudPointeurSurPoint nd3 = (CNoeudPointeurSurPoint) collectionPoints.get(i);
  		CPointAncetre D = nd3.pointeurSurPoint;
  		CNoeudPointeurSurPoint nd4 = (CNoeudPointeurSurPoint) collectionPoints.get(i+1);
  		CPointAncetre E = nd4.pointeurSurPoint;
  		if (Kernel.estInterieurATriangle(x, y, A.x, A.y, D.x, D.y, E.x, E.y)) {
				indicePremierSommet.setValue(i);
				// En appelant I le point d'intersection on calcule les coordonnes barycentriques du point
				// de coordonnes (x; y) par rapport au triangle ICD
				Kernel.coefficientsPointInterieur(x, y, A.x, A.y, D.x, D.y, E.x, E.y, alpha, beta, gamma);
				estSurCote.setValue(false);
				return true;
			}
  	}
  	return false;
  }
}
