/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.IOException;


import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.Kernel;
import mathgraph32.MtgFrame;
import types.DimensionFenetre;
import types.NatureCalcul;
import types.VecteurDirecteur;

public class CProduitScalaire extends CCalculAncetre {
	public double prosca;  // Le rsultat renvoy
	public CVecteur vect1;
	public CVecteur vect2;
	
	/** Constructeur par dfaut */
	public CProduitScalaire() {
	}
	
  /** Constructeur pour clonage */
  public CProduitScalaire(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      String nomInit, CVecteur vect1Init, CVecteur vect2Init) {
    super(liste, impProtoInit, estElementFinalInit, nomInit);
    vect1 = vect1Init;
    vect2 = vect2Init;
  }
	
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(vect1);
    int ind2 = listeSource.indexOf(vect2);   
    int ind3 = listeSource.indexOf(impProto);    
    CElementBase ptelb = new CProduitScalaire(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, nomCalcul, (CVecteur)(listeCible.get(ind1, CVecteur.class)),
      (CVecteur)(listeCible.get(ind2, CVecteur.class)));
    return ptelb;    
  }
  
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NMesureProduitScalaire;
  }
  
  @Override
  public String infoPourHistorique() {
    String ch = getString("chinfo157") + " " + vect1.point1.rendNom() + vect1.point2.rendNom() + " "
    	+ getString("chinfo158") + " " + vect2.point1.rendNom() + vect2.point2.rendNom() + "\n" + getString("chinfo159") + " " + nom();
    if (existe) ch = ch + "\n"	+ getString("chinfo97") + " " + Kernel.chaineNombre(prosca, 16);
    else ch = ch + "\n" + getString("ch18");
    return ch;
  }
  
  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(vect1.point1);
    liste.add(vect1.point2);
    liste.add(vect1);
    liste.add(vect2.point1);
    liste.add(vect2.point2);
    liste.add(vect2);
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || vect1.dependDe(p) || vect2.dependDe(p)
    || listeProprietaire.pointeurLongueurUnite.dependDe(p));
  }

  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    return super.dependDePourReclassement(p) || vect1.dependDePourReclassement(p) || vect2.dependDePourReclassement(p)
    ||(listeProprietaire.pointeurLongueurUnite.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || vect1.dependDePourBoucle(p) || vect2.dependDePourBoucle(p)
    ||(listeProprietaire.pointeurLongueurUnite.dependDePourBoucle(p));
  }  
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    CLongueur pLongUnite = listeProprietaire.pointeurLongueurUnite;
    existe = vect1.existe && vect2.existe && pLongUnite.existe;
    if (!existe) return;
    double longueurUnite = pLongUnite.rendLongueur();
    // Si la longueur unit est nulle, pLongUnite n'existe pas
    VecteurDirecteur v1 = new VecteurDirecteur(vect1.point1, vect1.point2);
    VecteurDirecteur v2 = new VecteurDirecteur(vect2.point1, vect2.point2);
    double n1 = v1.norme()/longueurUnite;
    double n2 = v2.norme()/longueurUnite;
    if ((n1 == 0) || (n2 == 0)) {
    	prosca = 0;
    	return;
    }
    double ang = v1.mesureAngleVecteurs(v2);
    prosca = n1*n2*Math.cos(ang);
  }
  
  @Override
  public double rendValeur(boolean infoRandom) {
    return prosca;
  }
  
  @Override
  public boolean confonduAvec(CElementBase p) {
    CProduitScalaire pt;
    if (p.getClass() == this.getClass()) {
      pt = (CProduitScalaire) p;
      return ((vect1 == pt.vect1) && (vect2 == pt.vect2)) || ((vect1 == pt.vect2) && (vect2 == pt.vect1));
    }
    else return false;
  }    
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		(listeOb.contains(vect1) || vect1.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(vect2) || vect2.estDefiniParObjetDans(listeOb));
	}  
  
  @Override
  public boolean nomIndispensable(CElementGraphique el) {
    return (el == vect1.point1) || (el == vect1.point2) || (el == vect2.point1) || (el == vect2.point2);
  }
  
  /** Lecture depuis dans un flux java */  
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    vect1 = (CVecteur) (list.get(ind1, CVecteur.class));
    vect2 = (CVecteur) (list.get(ind2, CVecteur.class));
  }  
  
  /** Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(vect1);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(vect2);
    oups.writeInt(ind2);
  }     
  
}
