/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.geom.*;
import java.io.*;

import types.*;

import dialogs.AngleRotDlg;
import dialogs.MtgDlg;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

import mathgraph32.MtgFrame;


public class CRotation extends CTransformation {
	CPointAncetre centre;
	public CValeurAngle angle; // Peut tre modifi par un dialogue donc public
  // Java : modification par rapport au C++ : on stocke la valeur numrique de l'angle
  // pour ne pas avoir  la recalculer plus tard
  double valeurAngle;

  /** Creates new CRotation */
  public CRotation() {
  }
  
  @Override
	public int natureTransformation() {
		return rotation;
	}
  
  // Conctructeur avec un angle constant non ditable pour les objets visuels
  public CRotation(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      CPointAncetre centreInit, double ang) {
    super(liste, impProtoInit, estElementFinalInit);
    angle = new CValeurAngle(liste, ang);
    centre = centreInit;
  }
  
  
  // Constructeur pour clonage
  public CRotation(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      CPointAncetre centreInit, CValeurAngle angleInit) {
    super(liste, impProtoInit, estElementFinalInit);
    centre = centreInit;
    angle = angleInit;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(centre);
    CValeurAngle angleClone = (CValeurAngle) angle.getClone(listeSource, listeCible);
    int ind2 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CRotation(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)), angleClone);
    return ptelb;
  }

  String designationTransf() {
    return getString("chinfo23");
  }  
  

  String complementPourHist() {
    return "\n" + getString("chinfo27") + " " + angle.chaineInfo();
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || centre.dependDe(p) || angle.dependDe(p));
  }

  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    return super.dependDePourReclassement(p) || centre.dependDePourReclassement(p) || angle.dependDePourReclassement(p);
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || centre.dependDePourBoucle(p) || angle.dependDePourBoucle(p));
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	angle.positionne(infoRandom, dimfen);
    existe = centre.existe && angle.existe();
    if (existe) valeurAngle = angle.rendValeurRadian(infoRandom);
  }

  public boolean confonduAvec(CElementBase p) {
    CRotation pt;
    if (p.getClass() == this.getClass()) {
      pt = (CRotation) p;
      return centre.confonduAvec(pt.centre) && angle.confonduAvec(pt.angle);
    } else return false;
  }
  
  public double transformeLongueur(double L) {
    return L;
  }

  public void transformePoint(double x, double y, Point2D.Double pointr) {
  
    VecteurDirecteur u = new VecteurDirecteur(centre.x, centre.y, x, y);
    VecteurDirecteur v = new VecteurDirecteur();
    u.tourne(valeurAngle,v);
    pointr.x = centre.x + v.x;
    pointr.y = centre.y + v.y;
  }

  public void transformeVecteur(VecteurDirecteur u, VecteurDirecteur ur) {
  	u.tourne(valeurAngle,ur);
  }

  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (centre == ancienPoint) centre = nouveauPoint;
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		(listeOb.contains(centre) || centre.estDefiniParObjetDans(listeOb))
		&&
		angle.estDefiniParObjetDans(listeOb);
	}
  
  /**
   * Fonction renvoyant true lorsque les images par ce type de transformation peuvent tre modifies
   * via un menu. C'est le cas pour les rotations, homothties, similitudes, inversions pour lequels
   * il faudra redfinir  cette fonction
   * @return
   */
  public boolean imageModifiableParMenu() {
  	return true;
  }
  
  /** Fonction renvoyant un dialogue seulement dans le cas o imageModifiableParMenu() renvoie true
   * Renverra le dialogue associ  la modification de la transformation.
   * Par exemple pour une rotation renverra un angleRotDlg.
   * @param image
   * @return
   */
  public MtgDlg dialogueModificationImage(MtgFrame frame) {
  	return new AngleRotDlg(frame, this, true);
  }
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    centre = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    angle = new CValeurAngle();
    angle.chargeDepuisFlux(inps, list);
  }  

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    centre = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    angle = new CValeurAngle();
    angle.read(inps, list);
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(centre);
    oups.writeInt(ind1);
    angle.write(oups, list);
  }    
}
