/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.geom.*;
import java.io.*;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CTranslation extends CTransformation {
	public CPointAncetre origineVecteur;
	public CPointAncetre extremiteVecteur;
  // Java : modification par rapport au C++ : on stocke la valeur numrique du vecteur
  // pour ne pas avoir  la recalculer plus tard
  VecteurDirecteur valeurVecteur;

  /** Creates new CTranslation */
  public CTranslation() {
    valeurVecteur = new VecteurDirecteur();
  }
  
  @Override
	public int natureTransformation() {
		return translation;
	}
  
  
  // Constructeur pour clonage
  public CTranslation(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    CPointAncetre origineVecteurInit, CPointAncetre extremiteVecteurInit) {
    super(liste, impProtoInit, estElementFinalInit);
    valeurVecteur = new VecteurDirecteur();
    origineVecteur = origineVecteurInit;
    extremiteVecteur= extremiteVecteurInit;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(origineVecteur);
    int ind2 = listeSource.indexOf(extremiteVecteur);
    int ind3 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CTranslation(listeCible, (CImplementationProto)listeCible.get(ind3, CImplementationProto.class),
      estElementFinal, (CPointAncetre)listeCible.get(ind1, CPointAncetre.class),
      (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)));
    return ptelb;    
  }

  @Override
  void AjouteElementsVisuelsPourHist(CListeObjets liste) {
    CVecteur vec = new CVecteur(liste, null, false,
      Couleur.TraduitCouleur(2), false, StyleTrait.traitEpais,
      origineVecteur, extremiteVecteur, StyleFleche.FlecheCourtePleine);
    liste.add(vec);
  }
  
  @Override
  String designationTransf() {
    return getString("chinfo19");
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || origineVecteur.dependDe(p) || extremiteVecteur.dependDe(p));
  }

  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    return super.dependDePourReclassement(p) || origineVecteur.dependDePourReclassement(p) || extremiteVecteur.dependDePourReclassement(p);
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || origineVecteur.dependDePourBoucle(p) || extremiteVecteur.dependDePourBoucle(p));
  }


  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = origineVecteur.existe && extremiteVecteur.existe;
    if (existe) valeurVecteur.setVecteur(origineVecteur, extremiteVecteur);
  }

  @Override
  public boolean confonduAvec(CElementBase p) {
    CTranslation pt;
    if (p.getClass() == this.getClass()) {
      pt = (CTranslation) p;
      return (origineVecteur == pt.origineVecteur)
              && (extremiteVecteur == pt.extremiteVecteur);
    } else return false;
  }
  
  @Override
  public double transformeLongueur(double L) {
    return L;
  }

  @Override
  public void transformePoint(double x, double y, Point2D.Double pointr) {
    pointr.x = x + valeurVecteur.x;
    pointr.y = y + valeurVecteur.y;
  }

  @Override
  public void transformeVecteur(VecteurDirecteur u, VecteurDirecteur ur) {
    u.setCopy(ur);
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		(listeOb.contains(origineVecteur) || origineVecteur.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(extremiteVecteur) || extremiteVecteur.estDefiniParObjetDans(listeOb));
	}  	      
  
  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (origineVecteur == ancienPoint) origineVecteur = nouveauPoint;
    if (extremiteVecteur == ancienPoint) extremiteVecteur = nouveauPoint;
  }
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    int ind2 = inps.readCInt();
    origineVecteur = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    extremiteVecteur = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    origineVecteur = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    extremiteVecteur = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(origineVecteur);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(extremiteVecteur);
    oups.writeInt(ind2);
  }
}
