/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Point;

import objets.*;
import types.Int;
import types.MotifPoint;
import types.NatureObjet;

import dialogs.RapportHomothetieDlg;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

/**
 * Outil de crtion d'images par une homothtie
 * @author Yves
 */
public class OutilHom extends Outil {
  CPointAncetre point;
  CElementGraphique objet;
  private int nombreImagesCrees;
  private CHomothetie ptHom;
  
  /** Creates a new instance of OutilRot */
  public OutilHom(MtgFrame frameInit) {
    super(frameInit, true);
  }
  
  /** Appel lorsque l'outil est slectionn */
  @Override
  public void select() {
    super.select();
    point = null;
    objet = null;
    nombreImagesCrees = 0;
    frame.outilPointageCre.designationDe(NatureObjet.NToutPoint);
    frame.outilPointageActif = frame.outilPointageCre;
    frame.indication(getString("homothetie1"));
  }
  
  /**
   * Il est ncessaire de regarder quand l'outil est dselectionn si on a vraiment cr des images
   * par une transformation. Sinon il faut supprimer le dernier objet qui est une homothtie
   */
  @Override
  public void deselect() {
    CListeObjets liste = frame.cadreActif().listePr;
    CElementBase elb = liste.get(liste.size() - 1);
    if (elb.getNature() == NatureObjet.NTransformation) {
    	frame.detruitDernierElement();
    }
    super.deselect();
  }

  @Override
  public void creeObjet() {
    CListeObjets liste = frame.cadreActif().listePr;
    if (nombreImagesCrees == 0) {
    	CTransformation hom = liste.transformationConfondueAvec(ptHom);
    	if (hom != null) {
    		ptHom = (CHomothetie) hom;
     	}
    	else frame.ajouteElement(ptHom);
    }
    Int nbImagesCrees = new Int();
    CElementGraphique elg = imageParTransformation(liste,  objet, ptHom, nbImagesCrees);
    if (frame.ajouteElementsImageAvecVerification(elg, nbImagesCrees.getValue(), ptHom)) {
    	frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil22"));
    	nombreImagesCrees++;
  		frame.cadreActif().repaintFigureAvecCalcul();    	
    }
  }
  
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    CListeObjets liste = frame.cadreActif().listePr;
    if (point == null) {
      frame.indication(getString("homothetie2"));
      point = (CPointAncetre) elg;
      excluDeDesignation(elg);
      ajouteClignotementDe(elg);
      resetClignotement();
      CValeurAssocieeAVariable rap = new CValeurAssocieeAVariable(liste, 0);
      ptHom = new CHomothetie(liste, null, false, point, rap);
      // On cre un dialogue modal permettant d'entrer l'angle de l'homothtie
      RapportHomothetieDlg rapportDlg = new RapportHomothetieDlg(frame, ptHom, false);
      // On redessine la figure pour liminer la dsignation ce point affiche pour le centre
      frame.cadreActif().videDesignation();
      frame.cadreActif().repaintFigureSansCalcul();
  		rapportDlg.setLocationRelativeTo(frame);
      rapportDlg.setVisible(true);
      // On regarde si la bote de dialogue a bien t valide par OK
      if (rapportDlg.retourOK()) {
        ajouteObjetsVisuels();
        frame.outilPointageActif.designationDe(NatureObjet.NToutPoint | NatureObjet.NTouteDroiteSaufVecteur |
                NatureObjet.NCercle | NatureObjet.NArc | NatureObjet.NPolygone
                | NatureObjet.NLigneBrisee);
        resetClignotement();
        frame.indication(getString("homothetie3"));
      }
      else {
        annuleClignotement();
        select(); 
      }      
    }
    else {
      objet = elg;
      creeObjet();
    }
  }      
  /* Mthode ajoutant les objets visuels  la liste listeObjetsVisuels de l'applet
   * au fur et  mesure que l'on dsigne les obejts  la souris
   * Doit tre redfini pour chaque outil utilisant une aide visuelle  la cration
   */
  @Override
  public void ajouteObjetsVisuels() {
  	MtgInternalFrame cadre = frame.cadreActif();
    CListeObjets liste = frame.listeObjetsVisuels;
    frame.ajouteObjetVisuel(ptHom);
    CPointImage ptim = new CPointImage(liste, null, false, frame.couleurActive(),
      MotifPoint.petitRond, 2, cadre.paneFigure.pointSouris, ptHom);
    frame.ajouteObjetVisuel(ptim);
  	// On cre un segment en pointills joignant le centre au point souris
  	CSegment pts1 = new CSegment(liste, null, false, point, ptim);
    frame.ajouteObjetVisuel(pts1);
    // Ajout version 4.5.2 : Si le rapport est ngatif on joint l'origine au point souris
    ptHom.positionne(cadre.infRand, cadre.dimf);
    if (ptHom.rapport.rendValeur(false) < 0) {
    	CSegment pts2 = new CSegment(liste, null, false, point, cadre.paneFigure.pointSouris);
      frame.ajouteObjetVisuel(pts2);
    }
  }
}
