/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Point;

import objets.CElementGraphique;
import objets.CLieuDiscretParPointLie;
import objets.CListeObjets;
import objets.CPointAncetre;
import objets.CPointLie;
import types.NatureObjet;

import dialogs.LieuDiscretDlg;

import mathgraph32.MtgFrame;

public class OutilLieuDiscretParPointLie extends Outil {
	private CPointAncetre pointPourTrace;
	private CPointLie pointLieGenerateur;
	
	/**
	 * Constructeur
	 * @param frameInit
	 */
  public OutilLieuDiscretParPointLie(MtgFrame frameInit) {
    super(frameInit, true);
  }
  
  @Override
  public void select() {
    super.select();
    pointPourTrace = null;
    pointLieGenerateur = null;
    frame.outilPointageCre.designationDe(NatureObjet.NToutPoint);
    frame.outilPointageActif = frame.outilPointageCre;
    frame.indication(getString("lieuParPointLie2"));
  }
  
  @Override
  public void deselect() {
    pointPourTrace = null;
    pointLieGenerateur = null;
    super.deselect();
  }
  
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
  	CListeObjets liste = frame.cadreActif().listePr;
    if (pointPourTrace == null) {
      pointPourTrace = (CPointAncetre) elg;
      // On n'exclut pas pointPourTrace d'une dsignation possible car le point li peut tre gal
      // au point gnrateur du lieu
      ajouteClignotementDe(elg);
      frame.indication(getString("lieuParPointLie2"));
      frame.listeClignotante.ajoutePointsLiesVisiblesDontDepend(liste,pointPourTrace);
      if (frame.listeClignotante.size() == 1) {
      	if (elg.getNature() != NatureObjet.NPointLie) {
	      	frame.dialogueAvertissement(getString("ch99"));
	      	frame.activeOutilCapture();
      	}
      }
      frame.outilPointageActif = frame.outilPointageObjetClignotant;
      frame.outilPointageActif.designationDe(NatureObjet.NPointLie);
      frame.outilPointageObjetClignotant.reset();
      resetClignotement();      
    }
    else {
      if (pointLieGenerateur == null) {
        pointLieGenerateur = (CPointLie) elg;
        creeObjet();
       }
    }
  }  

  @Override     
  public void creeObjet() {
    CLieuDiscretParPointLie pt = new CLieuDiscretParPointLie(frame.cadreActif().listePr, null,
       false, frame.couleurActive(), false, pointPourTrace, 200, frame.stylePointActif(), pointLieGenerateur);
    frame.cadreActif().videDesignation();
    LieuDiscretDlg dlg = new LieuDiscretDlg(frame, pt);
		dlg.setLocationRelativeTo(frame);
    dlg.setVisible(true);
    if (dlg.retourOK()) {
	    if (frame.ajouteElementAvecVerification(pt)){
	    	pt.etablitListeElementsAncetres();
	      frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil60"));
	      frame.cadreActif().repaintFigureAvecCalcul();
	    }
    }
    annuleClignotement();
    frame.activeOutilCapture();
  }

}
