/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Point;

import objets.CElementGraphique;
import objets.CLieuObjetParPtLie;
import objets.CListeObjets;
import objets.CPointLie;
import objets.CValeurAssocieeAVariable;
import types.NatureObjet;

import dialogs.LieuObjetDlg;

import mathgraph32.MtgFrame;


public class OutilLieuObjetParPointLie extends Outil {
	private CElementGraphique objetPourTrace;
	private CPointLie pointLieGenerateur;

  public OutilLieuObjetParPointLie(MtgFrame frameInit) {
    super(frameInit, true);
  }
  
  @Override
  public void select() {
    super.select();
    objetPourTrace = null;
    pointLieGenerateur = null;
    frame.outilPointageCre.designationDe(NatureObjet.NToutObjetPourLieu | NatureObjet.NSurface);
    frame.listeExclusionDesignation.ajouteSurfacesNePouvantPasGenererLieuObjet(frame.cadreActif().listePr);
    frame.outilPointageActif = frame.outilPointageCre;
    frame.outilPointageActif.reset();
    frame.indication(getString("lieuObjetParPointLie1"));
  }

  @Override
  public void deselect() {
    objetPourTrace = null;
    pointLieGenerateur = null;
    super.deselect();
  }
  
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
  	CListeObjets liste = frame.cadreActif().listePr;
    if (objetPourTrace == null) {
    	annuleClignotement();
      objetPourTrace = (CElementGraphique) elg;
      // On n'exclut pas pointPourTrace d'une dsignation possible car le point li peut tre gal
      // au point gnrateur du lieu
      ajouteClignotementDe(elg);
      resetClignotement();
      frame.indication(getString("lieuObjetParPointLie2"));
      frame.listeClignotante.ajoutePointsLiesVisiblesDontDepend(liste,objetPourTrace);
      // Modifi version 4.5 pour qu'on puisse gnrer un lieu de point d'un point li gnr par lui-me^me
      /*
      if (frame.listeClignotante.size() == 1) {
      	frame.dialogueAvertissement(getString("ch100"));
      	frame.activeOutilCapture();
      }
      */
      if ((frame.listeClignotante.size() == 1) && !elg.estDeNature(NatureObjet.NPointLie)) {
      	frame.dialogueAvertissement(getString("ch100"));
      	frame.activeOutilCapture();
      }
      
      frame.outilPointageActif = frame.outilPointageObjetClignotant;
      frame.outilPointageActif.designationDe(NatureObjet.NPointLie);
      frame.outilPointageObjetClignotant.reset();
      resetClignotement();      
    }
    else {
      if (pointLieGenerateur == null) {
        pointLieGenerateur = (CPointLie) elg;
        creeObjet();
       }
    }
  }
  
  @Override     
  public void creeObjet() {
  	CValeurAssocieeAVariable nbPositions = new CValeurAssocieeAVariable(frame.cadreActif().listePr, 40);
    CLieuObjetParPtLie pt = new CLieuObjetParPtLie(frame.cadreActif().listePr, null,
       false, frame.couleurActive(), false, objetPourTrace, nbPositions, pointLieGenerateur);
    frame.cadreActif().videDesignation();
    LieuObjetDlg dlg = new LieuObjetDlg(frame, pt, false);
		dlg.setLocationRelativeTo(frame);
    dlg.setVisible(true);
    if (dlg.retourOK()) {
	    if (frame.ajouteElementAvecVerification(pt)){
	    	pt.prepareListeCopiesObjet();
	    	pt.etablitListeElementsAncetres();
	      frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil62"));
	      frame.cadreActif().repaintFigureAvecCalcul();
	    }
    }
    annuleClignotement();
    frame.activeOutilCapture();
  }


}
