/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;
import objets.*;
import types.*;
import mathgraph32.MtgFigurePanel;
import mathgraph32.MtgFrame;

/**
 *
 * @author Yves
 */
public class OutilMilieu extends OutilPar2pts {
  
  /** Creates a new instance of OutilMilieu */
  public OutilMilieu(MtgFrame frameInit) {
    super(frameInit);
  }

  @Override
  public void creeObjet() {
    CMilieu pt = new CMilieu(frame.cadreActif().listePr, null, false, frame.couleurActive(), false, 0d, 0d,
        false, frame.genereNomPourPoint(), frame.pref_indiceTaillePoliceNom, frame.stylePointActif(),
        point1, point2, false);    		
    if (frame.ajouteElementAvecVerification(pt))
      frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil12"));
    annuleClignotement();
    select();
  }
  
  /* Mthode ajoutant les objets visuels  la liste listeObjetsVisuels de l'applet
   * au fur et  mesure que l'on dsigne les obejts  la souris
   * Doit tre redfini pour chaque outil utilisant une aide visuelle  la cration
   */
  @Override
  public void ajouteObjetsVisuels() {
    CListeObjets liste = frame.listeObjetsVisuels;
    MtgFigurePanel pane = frame.cadreActif().paneFigure;
    CMilieu ptm = new CMilieu(liste, null, false, frame.couleurActive(),
      frame.stylePointActif, point1, pane.pointSouris);
    liste.add(ptm);
  	// On cre deux segments en pointills
  	CSegment pts1 = new CSegment(liste, null, false, point1, ptm);
    liste.add(pts1);
  	CSegment pts2 = new CSegment(liste, null, false, ptm, pane.pointSouris);
    liste.add(pts2);
    // On cre une marque de segment pour les deux
    CMarqueSegment ptMarque1 = new CMarqueSegment(liste, null, false,
      Couleur.TraduitCouleur(1), false, StyleTrait.traitEpais,
      MotifMarqueSegment.MarqueSimple, pts1);
    liste.add(ptMarque1); 
    CMarqueSegment ptMarque2 = new CMarqueSegment(liste, null, false,
      Couleur.TraduitCouleur(1), false, StyleTrait.traitEpais,
      MotifMarqueSegment.MarqueSimple, pts2);
    liste.add(ptMarque2);
  }
  @Override
  public String chaineIndication (int indEtape) {
    switch(indEtape) {
      case 1 : return getString("milieu1");
      case 2 : return getString("milieu2");
      default: return "";
    }
  }
}
