/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Point;

import objets.CElementGraphique;
import objets.CLieuDeBase;
import objets.CPointAncetre;
import objets.CSurfaceLieuDeuxPoints;
import types.NatureObjet;

import mathgraph32.MtgFrame;

/**
 * Outil de cration d'une surface dlimite par un lieu de poins relis et deux points
 * @author Yves
 *
 */
public class OutilSurfaceLieuDeuxPoints extends Outil {
	private CLieuDeBase lieu;
	private CPointAncetre point1;
	private CPointAncetre point2;
	
	public OutilSurfaceLieuDeuxPoints(MtgFrame frameInit) {
		super(frameInit, true);
	}

  @Override
	public void select() {
		super.select();
		lieu = null;
		point1 = null;
		point2 = null;
		frame.outilPointageActif = frame.outilPointageCre;
		frame.outilPointageActif.designationDe(NatureObjet.NLieu);
		frame.indication(getString("surfaceLieuDeuxPoints1"));
	}

	/*
	 * Appel lorsqu'un objet a t cliqu pour tre trait par l'outil par
	 * exemple le premier point pour crer une droite passant par deux points
	 */
	public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    if (lieu == null) {
      lieu = (CLieuDeBase) elg;
      ajouteClignotementDe(lieu);
      resetClignotement();
      frame.indication(getString("surfaceLieuDeuxPoints2"));
  		frame.outilPointageActif.designationDe(NatureObjet.NToutPoint);
    }
    else {
    	if (point1 == null) {
	    	point1 = (CPointAncetre) elg;
	      ajouteClignotementDe(point1);
	      resetClignotement();
	      frame.indication(getString("surfaceLieuDeuxPoints3"));
    	}
    	else {
    		point2 = (CPointAncetre) elg;
	    	creeObjet();    		
    	}
    }
	}

  @Override
	public void creeObjet() {
    CSurfaceLieuDeuxPoints pt = new CSurfaceLieuDeuxPoints(frame.cadreActif().listePr, null,
        false, frame.couleurActive(), false, frame.styleRemplissageActif(), lieu, point1, point2);
    if (frame.ajouteElementAvecVerification(pt))
       frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil38"));
   	annuleClignotement();
   	select();
  }
}
