/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package types;
/*
 * FonteNoms.java
 *
 * Created on 17 juin 2004, 16:59
 */

/**
 *
 * @author  Yves
 * @version 
 */
import java.awt.*;
// Classe qui par ses mthodes de classe perlet de grer es fontes de caractres//
public class FonteNom {
  static final public int Normal   = 0;
  static final public int Gras   = 1;
  static final public int Italique   = 2;

  /** Creates new FonteNoms */
    public FonteNom() {
    }
   // Mthode renvoyant la taille de caractres correspondant  l'indice de taille Indicede Taille//
    public static final int taille(int indiceDeTaille, double coef) {
      switch (indiceDeTaille)
      {
        case 0 : // Taille 8
          return (int)(coef*10);
        case 1 :	// Taille 9
          return (int)(coef*11);
        case 2 : // Taille 11
          return (int)(coef*13);
        case 3 : // Taille 12
          return (int)(coef*14);
        case 4 : // Taille 14
          return (int)(coef*17);
        // Ajouts version 4.7.5.2
        default :
        	return (int)(coef*(2*indiceDeTaille+9));
        // default : return (int)(coef*13);
      }
    }
    
    // Modification de la version java : on accepte les lettres grecques
  public static final boolean lettre(char car) {
    boolean res = ((car  >= 'a') && (car <= 'z')) || ((car >= 'A') && (car <= 'Z'));
    // Lettres grecques acceptes par le logiciel
    res = res || ((car >= 0x3b1) && (car <= (0x3b1 + 7)));
    res = res || (car == 0x3b1 + 10);
    res = res || (car == 0x3b1 + 11);
    res = res || (car == 0x3b1 + 13);
    res = res || ((car >= 0x3b1 + 15) && (car <= (0x3b1 + 19)));
    res = res || ((car >= 0x3b1 + 21) && (car <= (0x3b1 + 24)));
    res = res || ((car >= 0x391 + 2) && (car <= (0x391 + 3)));
    res = res || (car == 0x391 + 15);
    res = res || ((car >= 0x391 + 18) && (car <= (0x391 + 19)));
    res = res || ((car >= 0x391 + 21) && (car <= (0x391 + 24)));
    return res;
  }

  public static final boolean prime(char car) {
    return (car == '\'') || (car == '"');
  }
  public static final boolean chiffre(char car) {
    return (car >= '0') && (car <= '9');
  }
  
  // Mthode de classe renvoyant une fonte de la taille voulue //
  public static final Font getFonteNom(int indiceTailleNom, int styleFonte, double coef ) {
  	if (coef == -1) coef = 1;
    switch (styleFonte) {
      case Normal :
        return new Font("Serif", Font.PLAIN, taille(indiceTailleNom, coef));
      case Gras :
        return new Font("Serif", Font.BOLD, taille(indiceTailleNom, coef));
      case Italique :
      default :
        return new Font("Serif", Font.ITALIC, taille(indiceTailleNom, coef));
    }
  }
  
  public final static String premiersCaracteresNonChiffres(String ch) {
  	int len = ch.length();
  	int i = 0;
  	while (i < len) {
  		if (chiffre(ch.charAt(i))) break;
  		else i++;
  	}
  	if (i == len) return ch;
  	else return ch.substring(0, i);
  }
  
  // Nouveauts pour la version 1.8
  // Fonction renvoyant une taille de police infrieure  celle de IndiceTaille
  // pour crire en exposant ou en indice
   public static final int indiceTaillePourExposantOuIndice(int indiceTaille)
  {
    switch (indiceTaille)
    {
        case 0 : // Taille 8
          return 0;
        case 1 : // Taille 9
          return 0;
        case 2 : // Taille 11
          return 0;
        case 3 : // Taille 12
          return 1;
        case 4: // Taille 14
          return 2;
        case 5 : // Taille 16
        case 6 : // Taille 18
        case 7 : // Taille 20
        case 8 : // Taille 22
        default :
          return 2;
    }
  }
  // Nouveauts pour la version 1.8
  // Fonction renvoyant le dcalage en hauteur ou vers le bas lorsqu'on crit un exposant
  // ou un indice suivant l'indice de la taille de la police utilise
  // Dcalages augments par rapport au C++
  public static final int decalageLogiquePourExposantOuIndice(int indiceTaille, double coef)
  {
    switch (indiceTaille)
    {
      case 0 : // Taille 8
        return (int)(coef*3);
      case 1 : // Taille 9
        return (int)(coef*3);
      case 2 : // Taille 11
        return (int)(coef*3);
      case 3 : // Taille 12
        return (int)(coef*4);
      case 4: // Taille 14
        return (int)(coef*4);
      case 5 : // Taille 16
      case 6 : // Taille 18
      case 7 : // Taille 20
      case 8 : // Taille 22
      default :
        return (int)(coef*3);
  	}
  }  
}
